@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -213,6 +213,57 @@ check_result_e UI_Check_base::Run()
 
 //------------------------------------------------------------------------
 
+void Vertex_FindDanglers(selection_c& sel)
+{
+	sel.change_type(OBJ_VERTICES);
+
+	if (NumVertices == 0 || NumLineDefs == 0)
+		return;
+
+	byte * line_counts = new byte[NumVertices];
+
+	memset(line_counts, 0, NumVertices);
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		int v1 = L->start;
+		int v2 = L->end;
+
+		// dangling vertices are fine for lines setting inside a sector
+		// (i.e. with same sector on both sides)
+		if (L->TwoSided() && (L->WhatSector(SIDE_LEFT) == L->WhatSector(SIDE_RIGHT)))
+		{
+			line_counts[v1] = line_counts[v2] = 2;
+			continue;
+		}
+
+		if (line_counts[v1] < 2) line_counts[v1] += 1;
+		if (line_counts[v2] < 2) line_counts[v2] += 1;
+	}
+
+	for (int k = 0 ; k < NumVertices ; k++)
+	{
+		if (line_counts[k] == 1)
+			sel.set(k);
+	}
+
+	delete[] line_counts;
+}
+
+
+void Vertex_ShowDanglers()
+{
+	if (edit.mode != OBJ_VERTICES)
+		Editor_ChangeMode('v');
+
+	Vertex_FindDanglers(*edit.Selected);
+
+	GoToErrors();
+}
+
+
 struct vertex_X_CMP_pred
 {
 	inline bool operator() (int A, int B) const
@@ -329,13 +380,24 @@ void Vertex_RemoveUnused()
 }
 
 
+void Vertex_ShowUnused()
+{
+	if (edit.mode != OBJ_VERTICES)
+		Editor_ChangeMode('v');
+
+	Vertex_FindUnused(*edit.Selected);
+
+	GoToErrors();
+}
+
+
 //------------------------------------------------------------------------
 
 class UI_Check_Vertices : public UI_Check_base
 {
 public:
 	UI_Check_Vertices(bool all_mode) :
-		UI_Check_base(520, 186, all_mode, "Check : Vertices",
+		UI_Check_base(520, 224, all_mode, "Check : Vertices",
 				      "Vertex test results")
 	{ }
 
@@ -354,12 +416,27 @@ public:
 		dialog->user_action = CKR_Highlight;
 	}
 
+	static void action_show_unused(Fl_Widget *w, void *data)
+	{
+		UI_Check_Vertices *dialog = (UI_Check_Vertices *)data;
+		Vertex_ShowUnused();
+		dialog->user_action = CKR_Highlight;
+	}
+
 	static void action_remove(Fl_Widget *w, void *data)
 	{
 		UI_Check_Vertices *dialog = (UI_Check_Vertices *)data;
 		Vertex_RemoveUnused();
 		dialog->user_action = CKR_TookAction;
 	}
+
+	static void action_show_danglers(Fl_Widget *w, void *data)
+	{
+		UI_Check_Vertices *dialog = (UI_Check_Vertices *)data;
+		Vertex_ShowDanglers();
+		dialog->user_action = CKR_Highlight;
+	}
+
 };
 
 
@@ -371,6 +448,19 @@ check_result_e CHECK_Vertices(int min_severity = 0)
 
 	for (;;)
 	{
+		Vertex_FindDanglers(sel);
+
+		if (sel.empty())
+			dialog->AddLine("No dangling vertices");
+		else
+		{
+			sprintf(check_message, "%d dangling vertices", sel.count_obj());
+
+			dialog->AddLine(check_message, 2, 210,
+			                "Show",  &UI_Check_Vertices::action_show_danglers);
+		}
+
+
 		Vertex_FindOverlaps(sel);
 
 		if (sel.empty())
@@ -395,7 +485,8 @@ check_result_e CHECK_Vertices(int min_severity = 0)
 		{
 			sprintf(check_message, "%d unused vertices", sel.count_obj());
 
-			dialog->AddLine(check_message, 1, 170,
+			dialog->AddLine(check_message, 1, 210,
+			                "Show",   &UI_Check_Vertices::action_show_unused,
 			                "Remove", &UI_Check_Vertices::action_remove);
 		}
 
@@ -594,12 +685,24 @@ void Sectors_FindUnknown(selection_c& list, std::map<int, int>& types)
 
 	for (int n = 0 ; n < NumSectors ; n++)
 	{
-		const sectortype_t *info = M_GetSectorType(Sectors[n]->type);
+		int type_num = Sectors[n]->type;
+
+		if (type_num < 0 || type_num >= 2048)
+		{
+			bump_unknown_type(types, type_num);
+			list.set(n);
+			continue;
+		}
+
+		// Boom generalized sectors
+		if (game_info.gen_types)
+			type_num &= 31;
+
+		const sectortype_t *info = M_GetSectorType(type_num);
 
 		if (strncmp(info->desc, "UNKNOWN", 7) == 0)
 		{
-			bump_unknown_type(types, Sectors[n]->type);
-
+			bump_unknown_type(types, type_num);
 			list.set(n);
 		}
 	}
@@ -1290,7 +1393,7 @@ void Things_FindInVoid(selection_c& list)
 
 		Objid obj;
 
-		GetCurObject(obj, OBJ_SECTORS, x, y);
+		GetNearObject(obj, OBJ_SECTORS, x, y);
 
 		if (! obj.is_nil())
 			continue;
@@ -1309,7 +1412,7 @@ void Things_FindInVoid(selection_c& list)
 			int x2 = x + ((corner & 1) ? -4 : +4);
 			int y2 = y + ((corner & 2) ? -4 : +4);
 
-			GetCurObject(obj, OBJ_SECTORS, x2, y2);
+			GetNearObject(obj, OBJ_SECTORS, x2, y2);
 
 			if (obj.is_nil())
 				out_count++;
@@ -1370,69 +1473,89 @@ static void CollectBlockingThings(std::vector<int>& list,
 }
 
 
-static bool ThingStuckInBlocker(const Thing *T, int r, char group,
-                                std::vector<int>& blockers,
-                                std::vector<int>& sizes)
-{
-	/*
-	   andrewj: the DOOM movement code for monsters works by moving
-	   the actor by a stepping distance which is based on its 'speed'
-	   value.  The move is allowed when the *new position* has no
-	   blocking things or walls, which means that things can overlap
-	   a short distance and won't be stuck.
-	  
-	   Properly taking this into account requires knowing the speed of
-	   each individual monster, but we don't have that information here.
-	   Hence I've chosen a conservative value based on the speed of the
-	   slowest monster (8 units).
-	   
-	   TODO: make it either game config or user preference.
-	*/
+/*
+   andrewj: the DOOM movement code for monsters works by moving
+   the actor by a stepping distance which is based on its 'speed'
+   value.  The move is allowed when the *new position* has no
+   blocking things or walls, which means that things can overlap
+   a short distance and won't be stuck.
+  
+   Properly taking this into account requires knowing the speed of
+   each individual monster, but we don't have that information here.
+   Hence I've chosen a conservative value based on the speed of the
+   slowest monster (8 units).
+   
+   TODO: make it either game config or user preference.
+*/
 #define MONSTER_STEP_DIST  8
 
-	if (group == 'm')
-		r = MAX(4, r - MONSTER_STEP_DIST);
 
-	int x1 = T->x - r;
-	int y1 = T->y - r;
-	int x2 = T->x + r;
-	int y2 = T->y + r;
+static bool ThingStuckInThing(const Thing *T1, const thingtype_t *info1,
+							  const Thing *T2, const thingtype_t *info2)
+{
+	SYS_ASSERT(T1 != T2);
 
-	for (unsigned int k = 0 ; k < blockers.size() ; k++)
+	// require one thing to be a monster or player
+	bool is_actor1 = (info1->group == 'm' || info1->group == 'p');
+	bool is_actor2 = (info2->group == 'm' || info2->group == 'p');
+
+	if (! (is_actor1 || is_actor2))
+		return false;
+
+	// check if T1 is stuck in T2
+	int r1 = info1->radius;
+	int r2 = info2->radius;
+
+	if (info1->group == 'm' && info2->group != 'p')
+		r1 = MAX(4, r1 - MONSTER_STEP_DIST);
+
+	else if (info2->group == 'm' && info1->group != 'p')
+		r2 = MAX(4, r2 - MONSTER_STEP_DIST);
+
+	if (T1->x - r1 >= T2->x + r2) return false;
+	if (T1->y - r1 >= T2->y + r2) return false;
+
+	if (T1->x + r1 <= T2->x - r2) return false;
+	if (T1->y + r1 <= T2->y - r2) return false;
+
+	// teleporters and DM starts can safely overlap moving actors
+	if ((info1->flags & THINGDEF_TELEPT) && is_actor2) return false;
+	if ((info2->flags & THINGDEF_TELEPT) && is_actor1) return false;
+
+	// check skill bits, except for players
+	int opt1 = T1->options;
+	int opt2 = T2->options;
+
+	if (Level_format == MAPF_Hexen)
 	{
-		const Thing *T2 = Things[blockers[k]];
+		if (info1->group == 'p') opt1 |= 0x7E7;
+		if (info2->group == 'p') opt2 |= 0x7E7;
 
-		if (T2 == T)
-			continue;
+		// check skill bits
+		if ((opt1 & opt2 & 0x07) == 0) return false;
 
-		int r2 = sizes[k];
+		// check class bits
+		if ((opt1 & opt2 & 0xE0) == 0) return false;
 
-		if (x1 >= T2->x + r2) continue;
-		if (y1 >= T2->y + r2) continue;
+		// check game mode
+		if ((opt1 & opt2 & 0x700) == 0) return false;
+	}
+	else
+	{
+		// invert game-mode bits (MTF_Not_COOP etc)
+		opt1 ^= 0x70; opt2 ^= 0x70;
 
-		if (x2 <= T2->x - r2) continue;
-		if (y2 <= T2->y - r2) continue;
+		if (info1->group == 'p') opt1 |= 0x77;
+		if (info2->group == 'p') opt2 |= 0x77;
 
-		// check skill bits, except for players
+		// check skill bits
+		if ((opt1 & opt2 & 0x07) == 0) return false;
 
-		if (group != 'p')
-		{
-			if (((T->options & T2->options) & 7) == 0)
-				continue;
-
-			// check game-mode bits  [FIXME for HEXEN]
-			if (((~T->options & ~T2->options) & 0x70) == 0)
-				continue;
-
-			// HEXEN TODO: check player-class bits
-			//
-			// if (((T->options & T2->options) & 0xE0) == 0) continue
-		}
-
-		return true;
+		// check game mode
+		if ((opt1 & opt2 & 0x70) == 0) return false;
 	}
 
-	return false;
+	return true;
 }
 
 
@@ -1459,6 +1582,10 @@ static inline bool LD_is_blocking(const LineDef *L)
 
 static bool ThingStuckInWall(const Thing *T, int r, char group)
 {
+	// only check players and monsters
+	if (! (group == 'p' || group == 'm'))
+		return false;
+
 	if (group == 'm')
 		r = MAX(4, r - MONSTER_STEP_DIST);
 
@@ -1495,25 +1622,23 @@ void Things_FindStuckies(selection_c& list)
 
 	CollectBlockingThings(blockers, sizes);
 
-	for (int n = 0 ; n < NumThings ; n++)
+	for (int n = 0 ; n < (int)blockers.size() ; n++)
 	{
-		const Thing *T = Things[n];
+		const Thing *T = Things[blockers[n]];
 
 		const thingtype_t *info = M_GetThingType(T->type);
 
-		// only check players and monsters
-		if (! (info->group == 'p' || info->group == 'm'))
-			continue;
+		if (ThingStuckInWall(T, info->radius, info->group))
+			list.set(blockers[n]);
 
-		// skip certain actors (e.g. deathmatch starts) which are often
-		// placed overlapping a normal player start.
-		if (info->flags & THINGDEF_PASS)
-			continue;
-
-		if (ThingStuckInBlocker(T, info->radius, info->group, blockers, sizes) ||
-			ThingStuckInWall   (T, info->radius, info->group))
+		for (int n2 = n + 1 ; n2 < (int)blockers.size() ; n2++)
 		{
-			list.set(n);
+			const Thing *T2 = Things[blockers[n2]];
+
+			const thingtype_t *info2 = M_GetThingType(T2->type);
+
+			if (ThingStuckInThing(T, info, T2, info2))
+				list.set(blockers[n]);
 		}
 	}
 }
