@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -126,6 +126,68 @@ void LineDefs_ShowMissingRight()
 }
 
 
+void LineDefs_FindManualDoors(selection_c& lines)
+{
+	// find D1/DR manual doors on one-sided linedefs
+
+	lines.change_type(OBJ_LINEDEFS);
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (L->type <= 0)
+			continue;
+
+		if (L->left >= 0)
+			continue;
+
+		const linetype_t *info = M_GetLineType(L->type);
+		
+		if (info->desc[0] == 'D' &&
+			(info->desc[1] == '1' || info->desc[1] == 'R'))
+		{
+			lines.set(n);
+		}
+	}
+}
+
+
+void LineDefs_ShowManualDoors()
+{
+	if (edit.mode != OBJ_LINEDEFS)
+		Editor_ChangeMode('l');
+
+	LineDefs_FindManualDoors(*edit.Selected);
+
+	GoToErrors();
+}
+
+
+void LineDefs_FixManualDoors()
+{
+	BA_Begin();
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (L->type <= 0 || L->left >= 0)
+			continue;
+
+		const linetype_t *info = M_GetLineType(L->type);
+
+		if (info->desc[0] == 'D' &&
+			(info->desc[1] == '1' || info->desc[1] == 'R'))
+		{
+			BA_ChangeLD(n, LineDef::F_TYPE, 0);
+		}
+	}
+
+	BA_End();
+}
+
+
 void LineDefs_FindLackImpass(selection_c& lines)
 {
 	lines.change_type(OBJ_LINEDEFS);
@@ -237,11 +299,17 @@ void LineDefs_FindUnknown(selection_c& list, std::map<int, int>& types)
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
-		const linetype_t *info = M_GetLineType(LineDefs[n]->type);
+		int type_num = LineDefs[n]->type;
+
+		const linetype_t *info = M_GetLineType(type_num);
+
+		// Boom generalized line type?
+		if (game_info.gen_types && is_genline(type_num))
+			continue;
 
 		if (strncmp(info->desc, "UNKNOWN", 7) == 0)
 		{
-			bung_unknown_type(types, LineDefs[n]->type);
+			bung_unknown_type(types, type_num);
 
 			list.set(n);
 		}
@@ -569,7 +637,7 @@ class UI_Check_LineDefs : public UI_Check_base
 {
 public:
 	UI_Check_LineDefs(bool all_mode) :
-		UI_Check_base(530, 350, all_mode, "Check : LineDefs",
+		UI_Check_base(530, 370, all_mode, "Check : LineDefs",
 		              "LineDef test results")
 	{ }
 
@@ -596,6 +664,21 @@ public:
 	}
 
 
+	static void action_show_manual_doors(Fl_Widget *w, void *data)
+	{
+		UI_Check_LineDefs *dialog = (UI_Check_LineDefs *)data;
+		LineDefs_ShowManualDoors();
+		dialog->user_action = CKR_Highlight;
+	}
+
+	static void action_fix_manual_doors(Fl_Widget *w, void *data)
+	{
+		UI_Check_LineDefs *dialog = (UI_Check_LineDefs *)data;
+		LineDefs_FixManualDoors();
+		dialog->user_action = CKR_TookAction;
+	}
+
+
 	static void action_show_lack_impass(Fl_Widget *w, void *data)
 	{
 		UI_Check_LineDefs *dialog = (UI_Check_LineDefs *)data;
@@ -752,6 +835,20 @@ check_result_e CHECK_LineDefs(int min_severity)
 		}
 
 
+		LineDefs_FindManualDoors(sel);
+
+		if (sel.empty())
+			dialog->AddLine("No manual doors on 1S linedefs");
+		else
+		{
+			sprintf(check_buffer, "%d manual doors on 1S linedefs", sel.count_obj());
+
+			dialog->AddLine(check_buffer, 2, 250,
+			                "Show", &UI_Check_LineDefs::action_show_manual_doors,
+			                "Fix",  &UI_Check_LineDefs::action_fix_manual_doors);
+		}
+
+
 		LineDefs_FindLackImpass(sel);
 
 		if (sel.empty())
@@ -927,7 +1024,7 @@ void CMD_ApplyTag()
 	Tags_ApplyNewValue(new_tag);
 
 	if (unselect)
-		edit.Selected->clear_all();
+		Selection_Clear(true);
 }
 
 
@@ -1195,6 +1292,20 @@ check_result_e CHECK_Tags(int min_severity)
 //------------------------------------------------------------------------
 
 
+static void bump_unknown_name(std::map<std::string, int>& list,
+                              const char *name)
+{
+	std::string t_name = name;
+
+	int count = 0;
+
+	if (list.find(t_name) != list.end())
+		count = list[t_name];
+
+	list[t_name] = count + 1;
+}
+
+
 static inline bool is_missing(const char *tex)
 {
 	return (tex[0] == 0 || tex[0] == '-');
@@ -1300,20 +1411,257 @@ void Textures_FixMissing()
 }
 
 
-static void bump_unknown_name(std::map<std::string, int>& list,
-                              const char *name)
+static bool is_transparent(const char *tex)
 {
-	std::string t_name = name;
+	// ignore lack of texture here
+	if (tex[0] == 0 || tex[0] == '-')
+		return false;
 
-	int count = 0;
+	Img_c *img = W_GetTexture(tex);
+	if (! img)
+		return false;
 
-	if (list.find(t_name) != list.end())
-		count = list[t_name];
-
-	list[t_name] = count + 1;
+	// note : this is slow
+	return img->has_transparent();
 }
 
 
+static int check_transparent(const char *tex,
+                             std::map<std::string, int>& names)
+{
+	if (is_transparent(tex))
+	{
+		bump_unknown_name(names, tex);
+		return 1;
+	}
+
+	return 0;
+}
+
+
+void Textures_FindTransparent(selection_c& lines,
+                              std::map<std::string, int>& names)
+{
+	lines.change_type(OBJ_LINEDEFS);
+
+	names.clear();
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (L->right < 0)
+			continue;
+
+		if (L->OneSided())
+		{
+			if (check_transparent(L->Right()->MidTex(), names))
+				lines.set(n);
+		}
+		else  // Two Sided
+		{
+			// note : plain OR operator here to check all parts (do NOT want short-circuit)
+			if (check_transparent(L->Right()->LowerTex(), names) |
+				check_transparent(L->Right()->UpperTex(), names) |
+				check_transparent(L-> Left()->LowerTex(), names) |
+				check_transparent(L-> Left()->UpperTex(), names))
+			{
+				lines.set(n);
+			}
+		}
+	}
+}
+
+
+void Textures_ShowTransparent()
+{
+	if (edit.mode != OBJ_LINEDEFS)
+		Editor_ChangeMode('l');
+
+	std::map<std::string, int> names;
+
+	Textures_FindTransparent(*edit.Selected, names);
+
+	GoToErrors();
+}
+
+
+void Textures_FixTransparent()
+{
+	int new_upper = BA_InternaliseString(default_upper_tex);
+	int new_mid   = BA_InternaliseString(default_mid_tex);
+	int new_lower = BA_InternaliseString(default_lower_tex);
+
+	BA_Begin();
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (L->right < 0)
+			continue;
+
+		if (L->OneSided())
+		{
+			if (is_transparent(L->Right()->MidTex()))
+				BA_ChangeSD(L->right, SideDef::F_MID_TEX, new_mid);
+		}
+		else  // Two Sided
+		{
+			if (is_transparent(L->Left()->LowerTex()))
+				BA_ChangeSD(L->left, SideDef::F_LOWER_TEX, new_lower);
+
+			if (is_transparent(L->Left()->UpperTex()))
+				BA_ChangeSD(L->left, SideDef::F_UPPER_TEX, new_upper);
+
+			if (is_transparent(L->Right()->LowerTex()))
+				BA_ChangeSD(L->right, SideDef::F_LOWER_TEX, new_lower);
+
+			if (is_transparent(L->Right()->UpperTex()))
+				BA_ChangeSD(L->right, SideDef::F_UPPER_TEX, new_upper);
+		}
+	}
+
+	BA_End();
+}
+
+
+void Textures_LogTransparent()
+{
+	selection_c sel;
+
+	std::map<std::string, int> names;
+	std::map<std::string, int>::iterator IT;
+
+	Textures_FindTransparent(sel, names);
+
+	LogPrintf("\n");
+	LogPrintf("Transparent textures on solid walls:\n");
+	LogPrintf("{\n");
+
+	for (IT = names.begin() ; IT != names.end() ; IT++)
+		LogPrintf("  %-9s x %d\n", IT->first.c_str(), IT->second);
+
+	LogPrintf("}\n");
+
+	LogViewer_Open();
+}
+
+
+static bool is_medusa(const char *tex)
+{
+	// ignore lack of texture here
+	if (tex[0] == 0 || tex[0] == '-')
+		return false;
+
+	return W_TextureCausesMedusa(tex);
+}
+
+
+static int check_medusa(const char *tex,
+                        std::map<std::string, int>& names)
+{
+	if (is_medusa(tex))
+	{
+		bump_unknown_name(names, tex);
+		return 1;
+	}
+
+	return 0;
+}
+
+
+void Textures_FindMedusa(selection_c& lines,
+                         std::map<std::string, int>& names)
+{
+	lines.change_type(OBJ_LINEDEFS);
+
+	names.clear();
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (L->right < 0 || L->left < 0)
+			continue;
+
+		if (check_medusa(L->Right()->MidTex(), names) |  /* plain OR */
+			check_medusa(L-> Left()->MidTex(), names))
+		{
+			lines.set(n);
+		}
+	}
+}
+
+
+void Textures_ShowMedusa()
+{
+	if (edit.mode != OBJ_LINEDEFS)
+		Editor_ChangeMode('l');
+
+	std::map<std::string, int> names;
+
+	Textures_FindMedusa(*edit.Selected, names);
+
+	GoToErrors();
+}
+
+
+void Textures_RemoveMedusa()
+{
+	int null_tex = BA_InternaliseString("-");
+
+	std::map<std::string, int> names;
+
+	BA_Begin();
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (L->right < 0 || L->left < 0)
+			continue;
+
+		if (check_medusa(L->Right()->MidTex(), names))
+		{
+			BA_ChangeSD(L->right, SideDef::F_MID_TEX, null_tex);
+		}
+
+		if (check_medusa(L-> Left()->MidTex(), names))
+		{
+			BA_ChangeSD(L->left, SideDef::F_MID_TEX, null_tex);
+		}
+	}
+
+	BA_End();
+}
+
+
+void Textures_LogMedusa()
+{
+	selection_c sel;
+
+	std::map<std::string, int> names;
+	std::map<std::string, int>::iterator IT;
+
+	Textures_FindMedusa(sel, names);
+
+	LogPrintf("\n");
+	LogPrintf("Medusa effect textures:\n");
+	LogPrintf("{\n");
+
+	for (IT = names.begin() ; IT != names.end() ; IT++)
+		LogPrintf("  %-9s x %d\n", IT->first.c_str(), IT->second);
+
+	LogPrintf("}\n");
+
+	LogViewer_Open();
+}
+
+
+// FIXME:  is_unknown_tex   (allow '-' and '#....')
+
+
 void Textures_FindUnknownTex(selection_c& lines,
                              std::map<std::string, int>& names)
 {
@@ -1501,7 +1849,7 @@ class UI_Check_Textures : public UI_Check_base
 {
 public:
 	UI_Check_Textures(bool all_mode) :
-		UI_Check_base(520, 226, all_mode, "Check : Textures",
+		UI_Check_base(565, 286, all_mode, "Check : Textures",
 		              "Texture test results")
 	{ }
 
@@ -1563,6 +1911,50 @@ public:
 		Textures_FixMissing();
 		dialog->user_action = CKR_TookAction;
 	}
+
+
+	static void action_show_transparent(Fl_Widget *w, void *data)
+	{
+		UI_Check_Textures *dialog = (UI_Check_Textures *)data;
+		Textures_ShowTransparent();
+		dialog->user_action = CKR_Highlight;
+	}
+
+	static void action_fix_transparent(Fl_Widget *w, void *data)
+	{
+		UI_Check_Textures *dialog = (UI_Check_Textures *)data;
+		Textures_FixTransparent();
+		dialog->user_action = CKR_TookAction;
+	}
+
+	static void action_log_transparent(Fl_Widget *w, void *data)
+	{
+		UI_Check_Textures *dialog = (UI_Check_Textures *)data;
+		Textures_LogTransparent();
+		dialog->user_action = CKR_Highlight;
+	}
+
+
+	static void action_show_medusa(Fl_Widget *w, void *data)
+	{
+		UI_Check_Textures *dialog = (UI_Check_Textures *)data;
+		Textures_ShowMedusa();
+		dialog->user_action = CKR_Highlight;
+	}
+
+	static void action_remove_medusa(Fl_Widget *w, void *data)
+	{
+		UI_Check_Textures *dialog = (UI_Check_Textures *)data;
+		Textures_RemoveMedusa();
+		dialog->user_action = CKR_TookAction;
+	}
+
+	static void action_log_medusa(Fl_Widget *w, void *data)
+	{
+		UI_Check_Textures *dialog = (UI_Check_Textures *)data;
+		Textures_LogMedusa();
+		dialog->user_action = CKR_Highlight;
+	}
 };
 
 
@@ -1606,6 +1998,26 @@ check_result_e CHECK_Textures(int min_severity)
 		}
 
 
+		if (game_info.medusa_bug)
+		{
+			Textures_FindMedusa(sel, names);
+
+			if (sel.empty())
+				dialog->AddLine("No textures causing Medusa Effect");
+			else
+			{
+				sprintf(check_buffer, "%d Medusa textures", (int)names.size());
+
+				dialog->AddLine(check_buffer, 2, 200,
+								"Show", &UI_Check_Textures::action_show_medusa,
+								"Log",  &UI_Check_Textures::action_log_medusa,
+								"Fix",  &UI_Check_Textures::action_remove_medusa);
+			}
+		}
+
+		dialog->AddGap(10);
+
+
 		Textures_FindMissing(sel);
 
 		if (sel.empty())
@@ -1614,12 +2026,27 @@ check_result_e CHECK_Textures(int min_severity)
 		{
 			sprintf(check_buffer, "%d missing textures on walls", sel.count_obj());
 
-			dialog->AddLine(check_buffer, 1, 260,
+			dialog->AddLine(check_buffer, 1, 270,
 			                "Show", &UI_Check_Textures::action_show_missing,
 			                "Fix",  &UI_Check_Textures::action_fix_missing);
 		}
 
 
+		Textures_FindTransparent(sel, names);
+
+		if (sel.empty())
+			dialog->AddLine("No transparent textures on solids");
+		else
+		{
+			sprintf(check_buffer, "%d transparent textures on solids", sel.count_obj());
+
+			dialog->AddLine(check_buffer, 1, 270,
+			                "Show", &UI_Check_Textures::action_show_transparent,
+			                "Fix",  &UI_Check_Textures::action_fix_transparent,
+			                "Log",  &UI_Check_Textures::action_log_transparent);
+		}
+
+
 		if (dialog->WorstSeverity() < min_severity)
 		{
 			delete dialog;
