@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2009-2012 Andrew Apted
+//  Copyright (C) 2009-2016 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -31,6 +31,7 @@
 #include "objects.h"
 #include "r_grid.h"
 #include "w_rawdef.h"
+#include "x_hover.h"
 #include "x_loop.h"
 
 
@@ -481,6 +482,7 @@ bool CMD_Copy()
 
 //------------------------------------------------------------------------
 
+
 static void PasteGroupOfObjects(int pos_x, int pos_y)
 {
 	int cx, cy;
@@ -575,18 +577,7 @@ static void PasteGroupOfObjects(int pos_x, int pos_y)
 
 		// if the linedef lost a side, fix texturing
 		if (L->OneSided() && L->Right()->MidTex()[0] == '-')
-		{
-			int tex;
-
-			if (L->Right()->LowerTex()[0] != '-')
-				tex = L->Right()->lower_tex;
-			else if (L->Right()->UpperTex()[0] != '-')
-				tex = L->Right()->upper_tex;
-			else
-				tex = BA_InternaliseString(default_mid_tex);
-
-			BA_ChangeSD(L->right, SideDef::F_MID_TEX, tex);
-		}
+			LD_FixForLostSide(new_l);
 	}
 
 	for (i = 0 ; i < clip_board->things.size() ; i++)
@@ -614,8 +605,8 @@ static void ReselectGroup()
 		{
 			int count = (int)clip_board->things.size();
 
-			edit.error_mode = false;
-			edit.Selected->clear_all();
+			Selection_Clear();
+
 			edit.Selected->frob_range(NumThings - count, NumThings-1, BOP_ADD);
 		}
 		return;
@@ -656,8 +647,7 @@ static void ReselectGroup()
 		new_sel.frob_range(NumSectors - count, NumSectors-1, BOP_ADD);
 	}
 
-	edit.error_mode = false;
-	edit.Selected->clear_all();
+	Selection_Clear();
 
 	ConvertSelection(&new_sel, edit.Selected);
 }
@@ -767,7 +757,7 @@ void UnusedVertices(selection_c *lines, selection_c *result)
 		// we are interested in the lines we are NOT deleting
 		if (lines->get(n))
 			continue;
-	
+
 		const LineDef *L = LineDefs[n];
 
 		result->clear(L->start);
@@ -786,7 +776,7 @@ void UnusedSideDefs(selection_c *lines, selection_c *result)
 		// we are interested in the lines we are NOT deleting
 		if (lines->get(n))
 			continue;
-	
+
 		const LineDef *L = LineDefs[n];
 
 		if (L->Right()) result->clear(L->right);
@@ -816,6 +806,65 @@ void UnusedLineDefs(selection_c *sectors, selection_c *result)
 	}
 }
 
+void UnusedSectors(selection_c *verts, selection_c *lines, selection_c *result)
+{
+	SYS_ASSERT(verts->what_type() == OBJ_VERTICES);
+	SYS_ASSERT(lines->what_type() == OBJ_LINEDEFS);
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (lines->get(n) || verts->get(L->start) || verts->get(L->end))
+		{
+			if (L->WhatSector(SIDE_LEFT ) >= 0) result->set(L->WhatSector(SIDE_LEFT ));
+			if (L->WhatSector(SIDE_RIGHT) >= 0) result->set(L->WhatSector(SIDE_RIGHT));
+		}
+	}
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (lines->get(n) || verts->get(L->start) || verts->get(L->end))
+			continue;
+
+		for (int pass = 0 ; pass < 2 ; pass++)
+		{
+			int what_side = pass ? SIDE_LEFT : SIDE_RIGHT;
+			
+			int sec_num = L->WhatSector(what_side);
+
+			if (sec_num < 0)
+				continue;
+
+			// if already clear, skip it (prevent expensive tests below)
+			if (! result->get(sec_num))
+				continue;
+
+			// check if the linedef opposite this is being deleted, which
+			// means this linedef will get mucked up.  When all remaining
+			// lines are like this, we should consider the sector "unused"
+			// and let it be removed.
+
+			int opp_side;
+			int opp_ld = OppositeLineDef(n, what_side, &opp_side);
+
+			if (opp_ld >= 0)
+			{
+				const LineDef *L2 = LineDefs[opp_ld];
+
+				if ((L2->WhatSector(opp_side) == sec_num) &&
+					(L2->WhatSector(SIDE_LEFT) != L2->WhatSector(SIDE_RIGHT)) &&
+					(lines->get(opp_ld) || verts->get(L2->start) || verts->get(L2->end)))
+				continue;
+			}
+
+			result->clear(sec_num);
+		}
+	}
+}
+
 
 static void FixupLineDefs(selection_c *lines, selection_c *sectors)
 {
@@ -852,8 +901,11 @@ static void FixupLineDefs(selection_c *lines, selection_c *sectors)
 }
 
 
-static void DeleteVertex_MergeLineDefs(int v_num)
+static bool DeleteVertex_MergeLineDefs(int v_num)
 {
+	// returns true if OK, false if would create an overlapping linedef
+	// [ meaning the vertex should be deleted normally ]
+
 	// find the linedefs
 	int ld1 = -1;
 	int ld2 = -1;
@@ -879,25 +931,36 @@ static void DeleteVertex_MergeLineDefs(int v_num)
 	LineDef *L1 = LineDefs[ld1];
 	LineDef *L2 = LineDefs[ld2];
 
-	// TODO: if (Length(ld1) < Length(ld2)) std::swap(ld1, ld2);
+	// we merge L2 into L1, unless L1 is significantly shorter
+	if (L1->CalcLength() < L2->CalcLength() * 0.7)
+	{
+		std::swap(ld1, ld2);
+		std::swap(L1,  L2);
+	}
 
-	// determine the other remaining vertex
-	int v_other;
+	// determine the remaining vertices
+	int v1 = (L1->start == v_num) ? L1->end : L1->start;
+	int v2 = (L2->start == v_num) ? L2->end : L2->start;
 
-	if (L2->start == v_num)
-		v_other = L2->end;
-	else
-		v_other = L2->start;
+	// ensure we don't create two directly overlapping linedefs
+	if (LineDefAlreadyExists(v1, v2))
+		return false;
+
+	BA_Begin();
 
 	if (L1->start == v_num)
-		BA_ChangeLD(ld1, LineDef::F_START, v_other);
+		BA_ChangeLD(ld1, LineDef::F_START, v2);
 	else
-		BA_ChangeLD(ld1, LineDef::F_END, v_other);
+		BA_ChangeLD(ld1, LineDef::F_END, v2);
 
-	// FIXME: update X offsets on existing linedef
+	// NOT-DO: update X offsets on existing linedef
 
 	BA_Delete(OBJ_LINEDEFS, ld2);
 	BA_Delete(OBJ_VERTICES, v_num);
+
+	BA_End();
+
+	return true;
 }
 
 
@@ -917,41 +980,49 @@ void CMD_Delete(void)
 	selection_c vert_sel(OBJ_VERTICES);
 	selection_c side_sel(OBJ_SIDEDEFS);
 	selection_c line_sel(OBJ_LINEDEFS);
+	selection_c  sec_sel(OBJ_SECTORS);
+
+	switch (edit.mode)
+	{
+		case OBJ_VERTICES:
+			vert_sel.merge(list);
+			break;
+
+		case OBJ_LINEDEFS:
+			line_sel.merge(list);
+			break;
+
+		case OBJ_SECTORS:
+			sec_sel.merge(list);
+			break;
+
+		default: /* OBJ_THINGS */
+			BA_Begin();
+			DeleteObjects(&list);
+			BA_End();
+
+			goto success;
+	}
 
 	// special case for a single vertex connected to two linedef,
 	// we delete the vertex but merge the two linedefs.
-	if (list.what_type() == OBJ_VERTICES && list.count_obj() == 1)
+	if (edit.mode == OBJ_VERTICES && vert_sel.count_obj() == 1)
 	{
-		int v_num = list.find_first();
+		int v_num = vert_sel.find_first();
 		SYS_ASSERT(v_num >= 0);
 
 		if (VertexHowManyLineDefs(v_num) == 2)
 		{
-			BA_Begin();
+			if (DeleteVertex_MergeLineDefs(v_num))
+				goto success;
 
-			DeleteVertex_MergeLineDefs(v_num);
-
-			BA_End();
-
-			goto success;
+			// delete vertex normally
 		}
 	}
 
-	BA_Begin();
-
-	// delete things from each deleted sector
-	if (!keep_things && list.what_type() == OBJ_SECTORS)
+	if (!keep_unused && edit.mode == OBJ_SECTORS)
 	{
-		selection_c thing_sel(OBJ_THINGS);
-
-		ConvertSelection(&list, &thing_sel);
-
-		DeleteObjects(&thing_sel);
-	}
-
-	if (!keep_unused && list.what_type() == OBJ_SECTORS)
-	{
-		UnusedLineDefs(&list, &line_sel);
+		UnusedLineDefs(&sec_sel, &line_sel);
 
 		if (line_sel.notempty())
 		{
@@ -960,37 +1031,54 @@ void CMD_Delete(void)
 		}
 	}
 
-	// perform linedef fixups here
-	if (list.what_type() == OBJ_SECTORS)
+	if (!keep_unused && edit.mode == OBJ_LINEDEFS)
+	{
+		UnusedVertices(&line_sel, &vert_sel);
+	}
+
+	if (edit.mode == OBJ_VERTICES || edit.mode == OBJ_LINEDEFS)
+	{
+		UnusedSideDefs(&line_sel, &side_sel);
+		UnusedSectors(&vert_sel, &line_sel, &sec_sel);
+	}
+
+	BA_Begin();
+
+	// delete things from each deleted sector
+	if (!keep_things && sec_sel.notempty())
+	{
+		selection_c thing_sel(OBJ_THINGS);
+
+		ConvertSelection(&sec_sel, &thing_sel);
+		DeleteObjects(&thing_sel);
+	}
+
+	// perform linedef fixups here (when sectors get removed)
+	if (sec_sel.notempty())
 	{
 		selection_c fixups(OBJ_LINEDEFS);
 
-		ConvertSelection(&list, &fixups);
+		ConvertSelection(&sec_sel, &fixups);
 
 		// skip lines which will get deleted
 		fixups.unmerge(line_sel);
 
-		FixupLineDefs(&fixups, &list);
+		FixupLineDefs(&fixups, &sec_sel);
 	}
 
-	if (!keep_unused && list.what_type() == OBJ_LINEDEFS)
-	{
-		UnusedVertices(&list, &vert_sel);
-		UnusedSideDefs(&list, &side_sel);
-	}
-
-	DeleteObjects(&side_sel);
-
-	DeleteObjects(&list);
-
+	// actually delete stuff, in the correct order
 	DeleteObjects(&line_sel);
+	DeleteObjects(&side_sel);
 	DeleteObjects(&vert_sel);
+	DeleteObjects( &sec_sel);
 
 	BA_End();
 
 success:
-	edit.error_mode = false;
-	edit.Selected->clear_all();
+	Editor_ClearAction();
+
+	Selection_Clear();
+
 	edit.highlight.clear();
 	edit.split_line.clear();
 
