@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2013 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -356,24 +356,6 @@ static side_on_a_line_t DetermineAdjoiner(side_on_a_line_t cur, char part,
 }
 
 
-#if 0  // TEST CRUD
-int TestAdjoinerLineDef(int ld)
-{
-	side_on_a_line_t zz = soal_make(ld, SIDE_RIGHT);
-
-	if (soal_sd(zz) < 0)
-		return -1;
-
-	side_on_a_line_t result = DetermineAdjoiner(zz, 0, 0);
-
-	if (result >= 0)
-		return soal_ld(result);
-
-	return -1;
-}
-#endif
-
-
 static int GetTextureHeight(const char *name)
 {
 	if (name[0] == '-')
@@ -631,14 +613,18 @@ void LineDefs_Align(int ld, int side, int sd, char part, int align_flags)
 //------------------------------------------------------------------------
 
 
-void FlipLineDef(int ld)
+static void FlipLine_verts(int ld)
 {
 	int old_start = LineDefs[ld]->start;
 	int old_end   = LineDefs[ld]->end;
 
 	BA_ChangeLD(ld, LineDef::F_START, old_end);
 	BA_ChangeLD(ld, LineDef::F_END, old_start);
+}
 
+
+static void FlipLine_sides(int ld)
+{
 	int old_right = LineDefs[ld]->right;
 	int old_left  = LineDefs[ld]->left;
 
@@ -647,6 +633,24 @@ void FlipLineDef(int ld)
 }
 
 
+void FlipLineDef(int ld)
+{
+	FlipLine_verts(ld);
+	FlipLine_sides(ld);
+}
+
+
+void FlipLineDef_safe(int ld)
+{
+	// this avoids creating a linedef with only a left side (no right)
+
+	FlipLine_verts(ld);
+
+	if (! LineDefs[ld]->OneSided())
+		FlipLine_sides(ld);
+}
+
+
 void FlipLineDefGroup(selection_c& flip)
 {
 	selection_iterator_c it;
@@ -673,7 +677,23 @@ void LIN_Flip(void)
 
 	BA_Begin();
 
-	FlipLineDefGroup(list);
+	bool do_verts = Exec_HasFlag("/verts");
+	bool do_sides = Exec_HasFlag("/sides");
+
+	selection_iterator_c it;
+
+	for (list.begin(&it) ; !it.at_end() ; ++it)
+	{
+		if (! do_verts && ! do_sides)
+		{
+			FlipLineDef_safe(*it);
+		}
+		else
+		{
+			if (do_verts) FlipLine_verts(*it);
+			if (do_sides) FlipLine_sides(*it);
+		}
+	}
 
 	BA_End();
 }
@@ -778,7 +798,7 @@ void LIN_SplitHalf(void)
 
 	// clear current selection, since the size needs to grow due to
 	// new linedefs being added to the map.
-	edit.Selected->clear_all();
+	Selection_Clear(true);
 
 	int new_first = NumLineDefs;
 	int new_count = 0;
@@ -823,7 +843,7 @@ void LD_MergedSecondSideDef(int ld)
 
 	BA_ChangeLD(ld, LineDef::F_FLAGS, new_flags);
 
-	// FIXME: make this a global pseudo-constant
+	// TODO: make this a global pseudo-constant
 	int null_tex = BA_InternaliseString("-");
 
 	// determine textures for each side
@@ -884,7 +904,7 @@ void LIN_MergeTwo(void)
 		return;
 	}
 
-	edit.Selected->clear_all();
+	Selection_Clear(true);
 
 
 	BA_Begin();
@@ -1053,27 +1073,23 @@ void LineDefs_SetLength(int new_len)
 }
 
 
-#if 0  // FIXME: MakeRectangularNook
-
-/*
- *  MakeRectangularNook - Make a nook or boss in a wall
- *  
- *  Before :    After :
- *                                  ^--->
- *                                  |   |
- *  +----------------->     +------->   v------->
- *      1st sidedef             1st sidedef
- *
- *  The length of the sides of the nook is sidelen.
- *  This is true when convex is false. If convex is true, the nook
- *  is actually a bump when viewed from the 1st sidedef.
- */
-void MakeRectangularNook (SelPtr obj, int width, int depth, int convex)
+void LD_FixForLostSide(int ld)
 {
-	....
-}
+	LineDef * L = LineDefs[ld];
 
-#endif
+	SYS_ASSERT(L->Right());
+
+	int tex;
+
+	if (L->Right()->LowerTex()[0] != '-')
+		tex = L->Right()->lower_tex;
+	else if (L->Right()->UpperTex()[0] != '-')
+		tex = L->Right()->upper_tex;
+	else
+		tex = BA_InternaliseString(default_mid_tex);
+
+	BA_ChangeSD(L->right, SideDef::F_MID_TEX, tex);
+}
 
 
 //--- editor settings ---
