@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -34,6 +34,7 @@
 #include "levels.h"
 #include "objects.h"
 #include "w_rawdef.h"
+#include "x_hover.h"
 #include "x_mirror.h"
 
 #include <algorithm>
@@ -51,7 +52,7 @@ int Vertex_FindExact(int x, int y)
 }
 
 
-// FIXME: InsertPolygonVertices
+// TODO: InsertPolygonVertices
 #if 0
 
 /*
@@ -73,7 +74,61 @@ void InsertPolygonVertices (int centerx, int centery, int sides, int radius)
  */
 static void MergeConnectedLines(int ld1, int ld2, int v)
 {
-	// TODO: review if we should e.g. muck around with sidedefs
+	LineDef *L1 = LineDefs[ld1];
+	LineDef *L2 = LineDefs[ld2];
+
+	bool ld1_onesided = L1->OneSided();
+	bool ld2_onesided = L2->OneSided();
+
+	int new_mid_tex = (ld1_onesided) ? L1->Right()->mid_tex :
+					  (ld2_onesided) ? L2->Right()->mid_tex : 0;
+
+	// flip ld1 so it would be parallel (after merging the other endpoints)
+	// with ld2 but going the opposite direction.
+	if ((L2->end == v) == (L1->end == v))
+	{
+		FlipLineDef(ld1);
+	}
+
+	bool same_left  = (L2->WhatSector(SIDE_LEFT)  == L1->WhatSector(SIDE_LEFT));
+	bool same_right = (L2->WhatSector(SIDE_RIGHT) == L1->WhatSector(SIDE_RIGHT));
+
+	if (same_left && same_right)
+	{
+		// delete other line too
+		// [ MUST do the highest numbered first ]
+		if (ld2 < ld1)
+			std::swap(ld1, ld2);
+
+		BA_Delete(OBJ_LINEDEFS, ld2);
+		BA_Delete(OBJ_LINEDEFS, ld1);
+		return;
+	}
+
+	if (same_left)
+	{
+		BA_ChangeLD(ld2, LineDef::F_LEFT, L1->right);
+	}
+	else if (same_right)
+	{
+		BA_ChangeLD(ld2, LineDef::F_RIGHT, L1->left);
+	}
+	else
+	{
+		// geometry was broken / unclosed sector(s)
+	}
+
+	// fix orientation of remaining linedef if needed
+	if (L2->Left() && ! L2->Right())
+	{
+		FlipLineDef(ld2);
+	}
+
+	if (L2->OneSided() && new_mid_tex > 0)
+	{
+		BA_ChangeSD(L2->right, SideDef::F_MID_TEX, new_mid_tex);
+	}
+
 
 	BA_Delete(OBJ_LINEDEFS, ld1);
 }
@@ -81,6 +136,8 @@ static void MergeConnectedLines(int ld1, int ld2, int v)
 
 void MergeVertex(int v1, int v2, bool v1_will_be_deleted)
 {
+	/* merge v1 into v2 */
+
 	SYS_ASSERT(v1 >= 0 && v2 >= 0);
 	SYS_ASSERT(v1 != v2);
 
@@ -113,8 +170,9 @@ void MergeVertex(int v1, int v2, bool v1_will_be_deleted)
 
 		if (found >= 0)
 		{
-			// this deletes linedef [n]
+			// this deletes linedef [n], and maybe the other too
 			MergeConnectedLines(n, found, v3);
+			break;
 		}
 	}
 
@@ -249,14 +307,18 @@ void Vertex_MergeList(selection_c *list)
 	if (list->count_obj() < 2)
 		return;
 
-	// the first vertex is kept (but moved to the middle coordinate),
-	// all the other vertices are removed.
+	// the first vertex is kept, all the other vertices are removed.
+
+	int v = list->find_first();
 
 	int new_x, new_y;
 
+#if 0
 	Objs_CalcMiddle(list, &new_x, &new_y);
-
-	int v = list->find_first();
+#else
+	new_x = Vertices[v]->x;
+	new_y = Vertices[v]->y;
+#endif
 
 	list->clear(v);
 
@@ -294,6 +356,8 @@ void VT_Merge()
 	}
 
 	Vertex_MergeList(edit.Selected);
+
+	Editor_ClearAction();
 }
 
 
@@ -336,7 +400,105 @@ void VT_Disconnect(void)
 
 	BA_End();
 
-	edit.Selected->clear_all();
+	Selection_Clear(true);
+}
+
+
+bool Vertex_TryFixDangler(int v_num)
+{
+	// see if this vertex is sitting on another one (or very close to it)
+	int v_other  = -1;
+	int max_dist = 2;
+
+	for (int i = 0 ; i < NumVertices ; i++)
+	{
+		if (i == v_num)
+			continue;
+
+		int dx = Vertices[v_num]->x - Vertices[i]->x;
+		int dy = Vertices[v_num]->y - Vertices[i]->y;
+
+		if (abs(dx) <= max_dist && abs(dy) <= max_dist &&
+			! LineDefAlreadyExists(v_num, v_other))
+		{
+			v_other = i;
+			break;
+		}
+	}
+
+
+	// check for a dangling vertex
+	if (VertexHowManyLineDefs(v_num) != 1)
+	{
+		if (v_other >= 0 && VertexHowManyLineDefs(v_other) == 1)
+			std::swap(v_num, v_other);
+		else
+			return false;
+	}
+
+
+	if (v_other >= 0)
+	{
+		Selection_Clear(true /* no_save */);
+
+		// delete highest numbered one  [ so the other index remains valid ]
+		if (v_num < v_other)
+			std::swap(v_num, v_other);
+
+fprintf(stderr, "Vertex_TryFixDangler : merge vert %d onto %d\n", v_num, v_other);
+		BA_Begin();
+
+		MergeVertex(v_num, v_other, true /* v1_will_be_deleted */);
+
+		selection_c list(OBJ_VERTICES);
+		list.set(v_num);
+
+		DeleteObjects(&list);
+
+		BA_End();
+
+		edit.Selected->set(v_other);
+
+		Beep("Merged a dangling vertex");
+		return true;
+	}
+
+
+#if 0
+	// find the line joined to this vertex
+	int joined_ld = -1;
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		if (LineDefs[n]->TouchesVertex(v_num))
+		{
+			joined_ld = n;
+			break;
+		}
+	}
+
+	SYS_ASSERT(joined_ld >= 0);
+#endif
+
+
+	// see if vertex is sitting on a line
+
+	Objid line_ob;
+
+	GetSplitLineForDangler(line_ob, v_num);
+
+	if (! line_ob.valid())
+		return false;
+
+fprintf(stderr, "Vertex_TryFixDangler : split linedef %d with vert %d\n", line_ob.num, v_num);
+	BA_Begin();
+
+	SplitLineDefAtVertex(line_ob.num, v_num);
+
+	BA_End();
+
+	// no vertices were added or removed, hence can continue Insert_Vertex
+	return false;
 }
 
 
@@ -437,7 +599,7 @@ void LIN_Disconnect(void)
 		Beep("Nothing was disconnected");
 
 	if (unselect)
-		edit.Selected->clear_all();
+		Selection_Clear(true /* no save */);
 }
 
 
@@ -712,7 +874,7 @@ void SEC_Disconnect(void)
 	BA_End();
 
 	if (unselect)
-		edit.Selected->clear_all();
+		Selection_Clear(true);
 }
 
 
