@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -59,6 +59,14 @@ bool mouse_wheel_scrolls_map = false;
 bool same_mode_clears_selection = false; 
 
 int multi_select_modifier = 0;
+int minimum_drag_pixels = 5;
+
+int sector_render_default = (int)SREND_Floor;
+
+
+extern bool easier_drawing_mode;
+
+void Editor_MouseMotion(int x, int y, keycode_t mod);
 
 
 /*
@@ -89,39 +97,15 @@ static void zoom_fit()
 }
 
 
-static void UpdateSplitLine(int drag_vert = -1)
+void RedrawMap()
 {
-	edit.split_line.clear();
-
-	// usually disabled while dragging stuff
-	if (edit.action == ACT_DRAG && edit.drag_single_vertex < 0)
+	if (! main_win)
 		return;
 
-	// in vertex mode, see if there is a linedef which would be split by
-	// adding a new vertex
-
-	if (edit.mode == OBJ_VERTICES && edit.pointer_in_window &&
-	    edit.highlight.is_nil())
-	{
-		GetSplitLineDef(edit.split_line, edit.map_x, edit.map_y, edit.drag_single_vertex);
-
-		// NOTE: OK if the split line has one of its vertices selected
-		//       (that case is handled by Insert_Vertex)
-	}
-
-	if (edit.split_line.valid())
-	{
-		int new_x = grid.SnapX(edit.map_x);
-		int new_y = grid.SnapY(edit.map_y);
-
-		// match logic in Insert_Vertex
-		if (! grid.snap)
-			MoveCoordOntoLineDef(edit.split_line.num, &new_x, &new_y);
-
-		main_win->canvas->SplitLineSet(edit.split_line.num, new_x, new_y);
-	}
+	if (edit.render3d)
+		main_win->render->redraw();
 	else
-		main_win->canvas->SplitLineForget();
+		main_win->canvas->redraw();
 }
 
 
@@ -182,6 +166,42 @@ static void UpdatePanel()
 }
 
 
+static void UpdateSplitLine(int map_x, int map_y)
+{
+	edit.split_line.clear();
+
+	// usually disabled while dragging stuff
+	if (edit.action == ACT_DRAG && edit.drag_single_vertex < 0)
+		return;
+
+	// in vertex mode, see if there is a linedef which would be split by
+	// adding a new vertex
+
+	if (edit.mode == OBJ_VERTICES && edit.pointer_in_window &&
+	    edit.highlight.is_nil())
+	{
+		GetSplitLineDef(edit.split_line, map_x, map_y, edit.drag_single_vertex);
+
+		// NOTE: OK if the split line has one of its vertices selected
+		//       (that case is handled by Insert_Vertex)
+	}
+
+	if (edit.split_line.valid())
+	{
+		edit.split_x = grid.SnapX(map_x);
+		edit.split_y = grid.SnapY(map_y);
+
+		// in FREE mode, ensure the new vertex is directly on the linedef
+		if (! grid.snap)
+			MoveCoordOntoLineDef(edit.split_line.num, &edit.split_x, &edit.split_y);
+
+		main_win->canvas->SplitLineSet(edit.split_line.num, edit.split_x, edit.split_y);
+	}
+	else
+		main_win->canvas->SplitLineForget();
+}
+
+
 void UpdateHighlight()
 {
 	bool dragging = (edit.action == ACT_DRAG);
@@ -193,7 +213,7 @@ void UpdateHighlight()
 	if (edit.pointer_in_window &&
 	    (!dragging || edit.drag_single_vertex >= 0))
 	{
-		GetCurObject(edit.highlight, edit.mode, edit.map_x, edit.map_y);
+		GetNearObject(edit.highlight, edit.mode, edit.map_x, edit.map_y);
 
 		// guarantee that we cannot drag a vertex onto itself
 		if (edit.drag_single_vertex >= 0 && edit.highlight.valid() &&
@@ -204,14 +224,36 @@ void UpdateHighlight()
 	}
 
 
+	UpdateSplitLine(edit.map_x, edit.map_y);
+
+
+	// in drawing mode, highlight extends to a vertex at the snap position
+	if (edit.mode == OBJ_VERTICES && grid.snap &&
+		edit.action == ACT_DRAW_LINE &&
+		edit.highlight.is_nil() && edit.split_line.is_nil())
+	{
+		int new_x = grid.SnapX(edit.map_x);
+		int new_y = grid.SnapY(edit.map_y);
+
+		int near_vert = Vertex_FindExact(new_x, new_y);
+
+		if (near_vert >= 0)
+		{
+			edit.highlight = Objid(OBJ_VERTICES, near_vert);
+		}
+		else
+		{
+			UpdateSplitLine(new_x, new_y);
+		}
+	}
+
+
 	if (edit.highlight.valid())
 		main_win->canvas->HighlightSet(edit.highlight);
 	else
 		main_win->canvas->HighlightForget();
 
 
-	UpdateSplitLine();
-
 	UpdatePanel();
 }
 
@@ -242,9 +284,7 @@ void Editor_ClearErrorMode()
 {
 	if (edit.error_mode)
 	{
-		edit.error_mode = false;
-		edit.Selected->clear_all();
-		edit.RedrawMap = 1;
+		Selection_Clear();
 	}
 }
 
@@ -266,12 +306,12 @@ void Editor_ChangeMode_Raw(obj_type_e new_mode)
 }
 
 
-void Editor_ChangeMode(char mode)
+void Editor_ChangeMode(char mode_char)
 {
 	obj_type_e  prev_type = edit.mode;
 
 	// Set the object type according to the new mode.
-	switch (mode)
+	switch (mode_char)
 	{
 		case 't': Editor_ChangeMode_Raw(OBJ_THINGS);   break;
 		case 'l': Editor_ChangeMode_Raw(OBJ_LINEDEFS); break;
@@ -279,13 +319,15 @@ void Editor_ChangeMode(char mode)
 		case 'v': Editor_ChangeMode_Raw(OBJ_VERTICES); break;
 
 		default:
-			LogPrintf("INTERNAL ERROR: unknown mode %d\n", mode);
+			LogPrintf("INTERNAL ERROR: unknown mode %d\n", mode_char);
 			return;
 	}
 
 	if (prev_type != edit.mode)
 	{
-		main_win->NewEditMode(mode);
+		Selection_Push();
+
+		main_win->NewEditMode(edit.mode);
 
 		// convert the selection
 		selection_c *prev_sel = edit.Selected;
@@ -297,18 +339,17 @@ void Editor_ChangeMode(char mode)
 	else if (main_win->isSpecialPanelShown())
 	{
 		// same mode, but this removes the special panel
-		main_win->NewEditMode(mode);
+		main_win->NewEditMode(edit.mode);
 	}
 	// -AJA- Yadex (DEU?) would clear the selection if the mode didn't
 	//       change.  We optionally emulate that behavior here.
 	else if (same_mode_clears_selection)
 	{
-		edit.Selected->clear_all();
+		Selection_Clear();
 	}
 
 	UpdateHighlight();
-
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -338,11 +379,13 @@ void CMD_SelectAll(void)
 
 	int total = NumObjects(edit.mode);
 
+	Selection_Push();
+
 	edit.Selected->change_type(edit.mode);
 	edit.Selected->frob_range(0, total-1, BOP_ADD);
-	edit.RedrawMap = 1;
 
 	UpdateHighlight();
+	RedrawMap();
 }
 
 
@@ -350,11 +393,16 @@ void CMD_UnselectAll(void)
 {
 	Editor_ClearErrorMode();
 
-	edit.Selected->change_type(edit.mode);
-	edit.Selected->clear_all();
-	edit.RedrawMap = 1;
+	if (edit.action == ACT_DRAW_LINE ||
+		edit.action == ACT_SCALE)
+	{
+		Editor_ClearAction();
+	}
+
+	Selection_Clear();
 
 	UpdateHighlight();
+	RedrawMap();
 }
 
 
@@ -376,9 +424,9 @@ void CMD_InvertSelection(void)
 	}
 
 	edit.Selected->frob_range(0, total-1, BOP_TOGGLE);
-	edit.RedrawMap = 1;
 
 	UpdateHighlight();
+	RedrawMap();
 }
 
 
@@ -437,7 +485,7 @@ void CMD_SetVar(void)
 	else if (y_stricmp(var_name, "obj_nums") == 0)
 	{
 		edit.show_object_numbers = bool_val;
-		edit.RedrawMap = 1;
+		RedrawMap();
 	}
 	else   // TODO: "skills"
 	{
@@ -484,13 +532,13 @@ void CMD_ToggleVar(void)
 	else if (y_stricmp(var_name, "obj_nums") == 0)
 	{
 		edit.show_object_numbers = ! edit.show_object_numbers;
-		edit.RedrawMap = 1;
+		RedrawMap();
 	}
 	else if (y_stricmp(var_name, "skills") == 0)
 	{
 		active_wmask ^= 1;
 		active_when = active_wmask;
-		edit.RedrawMap = 1;
+		RedrawMap();
 	}
 	else
 	{
@@ -502,6 +550,14 @@ void CMD_ToggleVar(void)
 static int mouse_last_x;
 static int mouse_last_y;
 
+// screen position when LMB was pressed
+static int mouse_button1_x;
+static int mouse_button1_y;
+
+// map location when LMB was pressed
+static int button1_map_x;
+static int button1_map_y;
+
 
 void Editor_ClearAction()
 {
@@ -514,7 +570,6 @@ void Editor_ClearAction()
 			Status_Clear();
 			break;
 	
-		case ACT_SCROLL_MAP:
 		case ACT_ADJUST_OFS:
 			main_win->SetCursor(FL_CURSOR_DEFAULT);
 			break;
@@ -543,7 +598,6 @@ void Editor_SetAction(editor_action_e  new_action)
 			Status_Set("META...");
 			break;
 
-		case ACT_SCROLL_MAP:
 		case ACT_ADJUST_OFS:
 			mouse_last_x = Fl::event_x();
 			mouse_last_y = Fl::event_y();
@@ -575,7 +629,7 @@ void CMD_BrowserMode(void)
 	char mode = toupper(EXEC_Param[0][0]);
 
 	if (! (mode == 'L' || mode == 'S' || mode == 'O' ||
-	       mode == 'T' || mode == 'F'))
+	       mode == 'T' || mode == 'F' || mode == 'G'))
 	{
 		Beep("Unknown browser mode: %s", EXEC_Param[0]);
 		return;
@@ -646,6 +700,15 @@ void CMD_Scroll(void)
 	delta_y = delta_y * main_win->canvas->h() / 100.0 / grid.Scale;
 
 	grid.Scroll(delta_x, delta_y);
+
+	// certain actions need to be updated
+	if (edit.action == ACT_SELBOX || edit.action == ACT_DRAW_LINE ||
+		edit.action == ACT_DRAG)
+	{
+		int mod = Fl::event_state() & MOD_ALL_MASK;
+
+		Editor_MouseMotion(Fl::event_x(), Fl::event_y(), mod);
+	}
 }
 
 
@@ -767,7 +830,7 @@ void CMD_ZoomWholeMap(void)
 
 	zoom_fit();
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -796,7 +859,7 @@ void CMD_GoToCamera(void)
 	edit.map_x = x;
 	edit.map_y = y;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -827,7 +890,21 @@ void CMD_PlaceCamera(void)
 		main_win->redraw();
 	}
 
-	edit.RedrawMap = 1;
+	RedrawMap();
+}
+
+
+void CMD_Gamma(void)
+{
+	int delta = (atoi(EXEC_Param[0]) >= 0) ? +1 : -1;
+
+	usegamma = (usegamma + delta + 5) % 5;
+
+	W_UpdateGamma();
+
+	Status_Set("Gamma level %d", usegamma);
+
+	RedrawMap();
 }
 
 
@@ -854,14 +931,16 @@ static void Editor_ScrollMap(int mode, int dx = 0, int dy = 0)
 	// started?
 	if (mode < 0)
 	{
-		Editor_SetAction(ACT_SCROLL_MAP);
+		edit.is_scrolling = true;
+		main_win->SetCursor(FL_CURSOR_HAND);
 		return;
 	}
 
 	// finished?
 	if (mode > 0)
 	{
-		Editor_ClearAction();
+		edit.is_scrolling = false;
+		main_win->SetCursor(FL_CURSOR_DEFAULT);
 		return;
 	}
 
@@ -960,6 +1039,9 @@ int Editor_RawWheel(int event)
 	if (edit.action == ACT_WAIT_META)
 		Editor_ClearAction();
 
+	// ensure we zoom from correct place
+	main_win->canvas->PointerPos(&edit.map_x, &edit.map_y);
+
 	wheel_dx = Fl::event_dx();
 	wheel_dy = Fl::event_dy();
 
@@ -985,6 +1067,14 @@ int Editor_RawButton(int event)
 
 	bool down = (event == FL_PUSH);
 
+	// Hack Alert : this is required to support pressing two buttons at the
+	// same time.  Without this, FLTK does not send us the second button
+	// release event, because when the first button is released the "pushed"
+	// widget becomes NULL.
+
+	if (Fl::event_buttons() != 0)
+		Fl::pushed(main_win->canvas);
+
 	// start scrolling the map?  [or moving in 3D view]
 	if (button == 3)
 	{
@@ -1031,7 +1121,7 @@ int Editor_RawMouse(int event)
 	int dy = Fl::event_y() - mouse_last_y;
 
 
-	if (edit.action == ACT_SCROLL_MAP)
+	if (edit.is_scrolling)
 	{
 		Editor_ScrollMap(0, dx, dy);
 	}
@@ -1041,16 +1131,11 @@ int Editor_RawMouse(int event)
 	}
 	else if (edit.render3d)
 	{
-		Render3D_MouseMotion(Fl::event_x(), Fl::event_y(), mod, event == FL_DRAG);
+		Render3D_MouseMotion(Fl::event_x(), Fl::event_y(), mod);
 	}
 	else
 	{
-		int map_x, map_y;
-
-		main_win->canvas->PointerPos(&map_x, &map_y);
-
-		Editor_MouseMotion(Fl::event_x(), Fl::event_y(), mod,
-						   map_x, map_y, event == FL_DRAG);
+		Editor_MouseMotion(Fl::event_x(), Fl::event_y(), mod);
 	}
 
 	mouse_last_x = Fl::event_x();
@@ -1096,20 +1181,56 @@ void Editor_MousePress(keycode_t mod)
 	edit.button_down = 1;
 	edit.button_mod  = mod;
 
-	Objid object;      // The object under the pointer
+	// remember some state (for dragging)
+	mouse_button1_x = Fl::event_x();
+	mouse_button1_y = Fl::event_y();
 
-	GetCurObject(object, edit.mode, edit.map_x, edit.map_y);
+	button1_map_x = edit.map_x;
+	button1_map_y = edit.map_y;
 
-	edit.clicked = object;
+	// this is a special case, since we want to allow the new vertex
+	// from a split-line (when in in drawing mode) to be draggable.
+	// [ that is achieved by setting edit.clicked ]
+
+	if (easier_drawing_mode && edit.split_line.valid() &&
+		edit.action != ACT_DRAW_LINE)
+	{
+		Insert_Vertex_split(edit.split_line.num, edit.split_x, edit.split_y);
+		return;
+	}
+
+	if (edit.action == ACT_DRAW_LINE)
+	{
+		bool force_cont = (mod == MOD_SHIFT);
+		bool no_fill    = (mod == MOD_COMMAND);
+
+		Insert_Vertex(force_cont, no_fill, true /* is_button */);
+		return;
+	}
+
+	// find the object under the pointer.
+	GetNearObject(edit.clicked, edit.mode, edit.map_x, edit.map_y);
+
+	// inhibit in sector/linedef mode when SHIFT is pressed, to allow
+	// opening a selection box in places which are otherwise impossible.
+	if (mod == MOD_SHIFT && (edit.mode == OBJ_SECTORS || edit.mode == OBJ_LINEDEFS))
+	{
+		edit.clicked.clear();
+	}
 
 	// clicking on an empty space starts a new selection box.
-
-	if (object.is_nil())
+	if (edit.clicked.is_nil())
 	{
+		if (edit.did_a_move)
+			Selection_Clear();
+
 		Editor_SetAction(ACT_SELBOX);
 		main_win->canvas->SelboxBegin(edit.map_x, edit.map_y);
 		return;
 	}
+
+	// Note: drawing mode is activated on RELEASE...
+	//       (as the user may be trying to drag the vertex)
 }
 
 
@@ -1123,7 +1244,8 @@ void Editor_MouseRelease()
 	bool was_did_move = edit.did_a_move;
 	edit.did_a_move = false;
 
-	/* Releasing the button while dragging : drop the selection. */
+	// releasing the button while dragging : drop the selection.
+
 	if (edit.action == ACT_DRAG)
 	{
 		Editor_ClearAction();
@@ -1140,25 +1262,13 @@ void Editor_MouseRelease()
 		}
 
 		edit.drag_single_vertex = -1;
-		edit.RedrawMap = 1;
+		RedrawMap();
 		return;
 	}
 
-	// optional multi-select : require a certain modifier key
-	if (multi_select_modifier &&
-		edit.button_mod != (multi_select_modifier == 1 ? MOD_SHIFT : MOD_COMMAND))
-	{
-		was_did_move = true;
-	}
+	// releasing the button while there was a selection box
+	// causes all the objects within the box to be selected.
 
-	if (click_obj.valid() && was_did_move)
-	{
-		edit.Selected->clear_all();
-	}
-
-	/* Releasing the button while there was a selection box
-	   causes all the objects within the box to be selected.
-	 */
 	if (edit.action == ACT_SELBOX)
 	{
 		Editor_ClearAction();
@@ -1174,27 +1284,58 @@ void Editor_MouseRelease()
 			SelectObjectsInBox(edit.Selected, edit.mode, x1, y1, x2, y2);
 
 		UpdateHighlight();
-		edit.RedrawMap = 1;
+		RedrawMap();
 		return;
 	}
 
 
-	if (click_obj.is_nil())
+	// nothing needed while in drawing mode
+	if (edit.action == ACT_DRAW_LINE)
 		return;
 
-	Objid object;      // object under the pointer
 
-	GetCurObject(object, edit.mode, edit.map_x, edit.map_y);
-
-	/* select the object if unselected, and vice versa.
-	 */
-	if (object == click_obj)
+	// optional multi-select : require a certain modifier key
+	if (multi_select_modifier &&
+		edit.button_mod != (multi_select_modifier == 1 ? MOD_SHIFT : MOD_COMMAND))
 	{
+		was_did_move = true;
+	}
+
+
+	// handle a clicked-on object
+	// e.g. select the object if unselected, and vice versa.
+
+	if (click_obj.valid())
+	{
+		bool was_empty = edit.Selected->empty();
+
 		Editor_ClearErrorMode();
 
-		edit.Selected->toggle(object.num);
+		if (was_did_move)
+			Selection_Clear();
 
-		edit.RedrawMap = 1;
+		// check if pointing at the same object as before
+		Objid near_obj;
+
+		GetNearObject(near_obj, edit.mode, edit.map_x, edit.map_y);
+
+		if (near_obj != click_obj)
+			return;
+
+		// begin drawing mode (unless a modifier was pressed)
+		if (easier_drawing_mode && edit.mode == OBJ_VERTICES &&
+			was_empty && edit.button_mod == 0)
+		{
+			Editor_SetAction(ACT_DRAW_LINE);
+			edit.drawing_from = click_obj.num;
+			edit.Selected->set(click_obj.num);
+
+			RedrawMap();
+			return;
+		}
+
+		edit.Selected->toggle(click_obj.num);
+		RedrawMap();
 		return;
 	}
 }
@@ -1202,17 +1343,20 @@ void Editor_MouseRelease()
 
 void Editor_MiddlePress(keycode_t mod)
 {
-fprintf(stderr, "Editor_MiddlePress : button_down:%d mod:%04x\n", edit.button_down, mod);
 	if (edit.button_down & 1)  // allow 0 or 2
 		return;
 
+#if 0
 	// ability to insert stuff via the mouse
 	if (mod == 0)
 	{
 		EXEC_Param[0] = "";
+		EXEC_Flags[0] = "";
+
 		CMD_Insert();
 		return;
 	}
+#endif
 
 	if (edit.Selected->empty())
 	{
@@ -1247,7 +1391,7 @@ void Editor_MiddleRelease()
 
 		CMD_ScaleObjects2(param);
 
-		edit.RedrawMap = 1;
+		RedrawMap();
 	}
 }
 
@@ -1256,13 +1400,21 @@ void Editor_MiddleRelease()
 void Editor_LeaveWindow()
 {
 	edit.pointer_in_window = false;
-	
+
+	// this offers a handy way to get out of drawing mode
+	if (edit.action == ACT_DRAW_LINE)
+		Editor_ClearAction();
+
 	UpdateHighlight();
 }
 
 
-void Editor_MouseMotion(int x, int y, keycode_t mod, int map_x, int map_y, bool drag)
+void Editor_MouseMotion(int x, int y, keycode_t mod)
 {
+	int map_x, map_y;
+
+	main_win->canvas->PointerPos(&map_x, &map_y);
+
 	edit.map_x = map_x;
 	edit.map_y = map_y;
 	edit.pointer_in_window = true; // FIXME
@@ -1270,7 +1422,7 @@ void Editor_MouseMotion(int x, int y, keycode_t mod, int map_x, int map_y, bool
 	if (edit.pointer_in_window)
 		main_win->info_bar->SetMouse(edit.map_x, edit.map_y);
 
-	// fprintf(stderr, "MOUSE MOTION: %d,%d  map: %d,%d\n", x, y, edit.map_x, edit.map_y);
+//  fprintf(stderr, "MOUSE MOTION: %d,%d  map: %d,%d\n", x, y, edit.map_x, edit.map_y);
 
 	if (edit.action == ACT_SCALE)
 	{
@@ -1278,29 +1430,19 @@ void Editor_MouseMotion(int x, int y, keycode_t mod, int map_x, int map_y, bool
 		return;
 	}
 
-	if (! drag)
+	if (edit.action == ACT_DRAW_LINE)
 	{
 		UpdateHighlight();
+		main_win->canvas->redraw();
+		return;
 	}
 
-	/* Moving the pointer with the left button pressed
-	   and a selection box exists : move the second
-	   corner of the selection box.
-	*/
-	else if (edit.action == ACT_SELBOX)
+	if (edit.action == ACT_SELBOX)
 	{
-		if (edit.did_a_move)
-			edit.Selected->clear_all();
-
 		main_win->canvas->SelboxUpdate(edit.map_x, edit.map_y);
 		return;
 	}
 
-	/* Moving the pointer with the left button pressed
-	   but no selection box exists and [Ctrl] was not
-	   pressed when the button was pressed :
-	   drag the selection.
-	*/
 	if (edit.action == ACT_DRAG)
 	{
 		main_win->canvas->DragUpdate(edit.map_x, edit.map_y);
@@ -1310,16 +1452,19 @@ void Editor_MouseMotion(int x, int y, keycode_t mod, int map_x, int map_y, bool
 		return;
 	}
 
-	/*
-	   begin dragging?
-	   TODO: require pixel dist from click point to be >= THRESHHOLD
-	 */
-	if (edit.button_down == 1 && edit.clicked.valid())
+	//
+	// begin dragging?
+	//
+	int pixel_dx = Fl::event_x() - mouse_button1_x;
+	int pixel_dy = Fl::event_y() - mouse_button1_y;
+
+	if (edit.button_down == 1 && edit.clicked.valid() &&
+		MAX(abs(pixel_dx), abs(pixel_dy)) >= minimum_drag_pixels)
 	{
 		if (! edit.Selected->get(edit.clicked.num))
 		{
 			if (edit.did_a_move)
-				edit.Selected->clear_all();
+				Selection_Clear();
 
 			edit.Selected->set(edit.clicked.num);
 			edit.did_a_move = false;
@@ -1327,10 +1472,10 @@ void Editor_MouseMotion(int x, int y, keycode_t mod, int map_x, int map_y, bool
 
 		int focus_x, focus_y;
 
-		GetDragFocus(&focus_x, &focus_y, edit.map_x, edit.map_y);
+		GetDragFocus(&focus_x, &focus_y, button1_map_x, button1_map_y);
 
 		Editor_SetAction(ACT_DRAG);
-		main_win->canvas->DragBegin(focus_x, focus_y, edit.map_x, edit.map_y);
+		main_win->canvas->DragBegin(focus_x, focus_y, button1_map_x, button1_map_y);
 
 		// check for a single vertex
 		edit.drag_single_vertex = -1;
@@ -1341,17 +1486,20 @@ void Editor_MouseMotion(int x, int y, keycode_t mod, int map_x, int map_y, bool
 			SYS_ASSERT(edit.drag_single_vertex >= 0);
 		}
 
-		// forget the highlight
-		edit.highlight.clear();
-		main_win->canvas->HighlightForget();
+		UpdateHighlight();
 		return;
 	}
+
+
+	// in general, just update the highlight, split-line (etc)
+
+	UpdateHighlight();
 }
 
 
 void Editor_Resize(int is_width, int is_height)
 {
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -1379,7 +1527,7 @@ static editor_command_t  command_table[] =
  	{	"BrowserMode",
 		&CMD_BrowserMode,
 		/* flags */ NULL,
-		/* keywords */ "obj tex flat line sec"
+		/* keywords */ "obj tex flat line sec genline"
 	},
 
 	{	"Set",
@@ -1428,6 +1576,10 @@ static editor_command_t  command_table[] =
 		&CMD_InvertSelection
 	},
 
+	{	"LastSelection",
+		&CMD_LastSelection
+	},
+
 	{	"Scroll",
 		&CMD_Scroll
 	},
@@ -1466,7 +1618,7 @@ static editor_command_t  command_table[] =
 
 	{	"Insert",
 		&CMD_Insert,
-		/* flags */ "/new"
+		/* flags */ "/new /continue /nofill"
 	},
 
 	{	"Delete",
@@ -1507,12 +1659,17 @@ static editor_command_t  command_table[] =
 		&CMD_Quantize
 	},
 
+	{	"Gamma",
+		&CMD_Gamma
+	},
+
 	{	"CopyAndPaste",
 		&CMD_CopyAndPaste
 	},
 
 	{	"CopyProperties",
-		&CMD_CopyProperties
+		&CMD_CopyProperties,
+		/* flags */ "/reverse"
 	},
 
 	{	"ApplyTag",
@@ -1529,7 +1686,8 @@ static editor_command_t  command_table[] =
 	/* -------- linedef -------- */
 
 	{	"LIN_Flip",
-		&LIN_Flip
+		&LIN_Flip,
+		/* flags */ "/verts /sides"
 	},
 
 	{	"LIN_SplitHalf",
@@ -1618,10 +1776,13 @@ void Editor_Init()
 	}
 
 	edit.action = ACT_NOTHING;
+	edit.is_scrolling = false;
 
 	edit.render3d = false;
 	edit.error_mode = false;
 
+	edit.sector_render_mode = sector_render_default;
+
 	edit.show_object_numbers = false;
 	edit.show_things_squares = false;
 	edit.show_things_sprites = true;
@@ -1632,6 +1793,7 @@ void Editor_Init()
 
 	edit.highlight.clear();
 	edit.split_line.clear();
+	edit.drawing_from = -1;
 	edit.drag_single_vertex = -1;
 
 	edit.Selected = new selection_c(edit.mode);
@@ -1658,14 +1820,21 @@ bool Editor_ParseUser(const char ** tokens, int num_tok)
 	if (strcmp(tokens[0], "render_mode") == 0 && num_tok >= 2)
 	{
 		edit.render3d = atoi(tokens[1]);
-		edit.RedrawMap = 1;
+		RedrawMap();
+		return true;
+	}
+
+	if (strcmp(tokens[0], "sector_render_mode") == 0 && num_tok >= 2)
+	{
+		edit.sector_render_mode = atoi(tokens[1]);
+		RedrawMap();
 		return true;
 	}
 
 	if (strcmp(tokens[0], "show_object_numbers") == 0 && num_tok >= 2)
 	{
 		edit.show_object_numbers = atoi(tokens[1]);
-		edit.RedrawMap = 1;
+		RedrawMap();
 		return true;
 	}
 
@@ -1686,6 +1855,7 @@ void Editor_WriteUser(FILE *fp)
 	}
 
 	fprintf(fp, "render_mode %d\n", edit.render3d ? 1 : 0);
+	fprintf(fp, "sector_render_mode %d\n", edit.sector_render_mode);
 	fprintf(fp, "show_object_numbers %d\n", edit.show_object_numbers ? 1 : 0);
 }
 
