@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -261,6 +261,15 @@ static const opt_desc_t options[] =
 		&backup_max_space
 	},
 
+	{	"easier_drawing_mode",
+		0,
+		OPT_BOOLEAN,
+		"v",
+		"Easier line drawing using the LMB",
+		NULL,
+		&easier_drawing_mode
+	},
+
 	{	"browser_small_tex",
 		0,
 		OPT_BOOLEAN,
@@ -270,6 +279,15 @@ static const opt_desc_t options[] =
 		&browser_small_tex
 	},
 
+	{	"default_gamma",
+		0,
+		OPT_INTEGER,
+		"v",
+		"Default gamma for images and 3D view (0..4)",
+		NULL,
+		&usegamma
+	},
+
 	{	"default_grid_mode",
 		0,
 		OPT_INTEGER,
@@ -360,6 +378,33 @@ static const opt_desc_t options[] =
 		&dotty_point_col
 	},
 
+	{	"floor_bump_small",
+		0,
+		OPT_INTEGER,
+		"v",
+		"distance for '+' and '-' buttons in sector panel while SHIFT is pressed",
+		NULL,
+		&floor_bump_small
+	},
+
+	{	"floor_bump_medium",
+		0,
+		OPT_INTEGER,
+		"v",
+		"distance for '+' and '-' buttons in sector panel without any modifier keys",
+		NULL,
+		&floor_bump_medium
+	},
+
+	{	"floor_bump_large",
+		0,
+		OPT_INTEGER,
+		"v",
+		"distance for '+' and '-' buttons in sector panel while CTRL is pressed",
+		NULL,
+		&floor_bump_large
+	},
+
 	{	"glbsp_fast",
 		0,
 		OPT_BOOLEAN,
@@ -459,6 +504,33 @@ static const opt_desc_t options[] =
 		&leave_offsets_alone
 	},
 
+	{	"light_bump_small",
+		0,
+		OPT_INTEGER,
+		"v",
+		"light step for '+' and '-' buttons in sector panel while SHIFT is pressed",
+		NULL,
+		&light_bump_small
+	},
+
+	{	"light_bump_medium",
+		0,
+		OPT_INTEGER,
+		"v",
+		"light step for '+' and '-' buttons in sector panel without any modifier keys",
+		NULL,
+		&light_bump_medium
+	},
+
+	{	"light_bump_large",
+		0,
+		OPT_INTEGER,
+		"v",
+		"light step for '+' and '-' buttons in sector panel while CTRL is pressed",
+		NULL,
+		&light_bump_large
+	},
+
 	{	"map_scroll_bars",
 		0,
 		OPT_BOOLEAN,
@@ -468,6 +540,15 @@ static const opt_desc_t options[] =
 		&map_scroll_bars
 	},
 
+	{	"minimum_drag_pixels",
+		0,
+		OPT_INTEGER,
+		"v",
+		"Minimum distance to move mouse to drag an object (in pixels)",
+		NULL,
+		&minimum_drag_pixels
+	},
+
 	{	"mouse_wheel_scrolls_map",
 		0,
 		OPT_BOOLEAN,
@@ -540,13 +621,13 @@ static const opt_desc_t options[] =
 		&normal_small_col
 	},
 
-	{	"render_aspect_ratio",
+	{	"render_pixel_aspect",
 		0,
 		OPT_INTEGER,
 		"v",
-		"Aspect ratio for 3D view : 100 * width / height",
+		"Pixel aspect ratio for 3D view (100 * width / height)",
 		NULL,
-		&render_aspect_ratio
+		&render_pixel_aspect
 	},
 
 	{	"render_high_detail",
@@ -598,7 +679,7 @@ static const opt_desc_t options[] =
 		0,
 		OPT_INTEGER,
 		"v",
-		"Amp. of scrolling (% of screen size)",
+		"Amplitude of scrolling (% of screen size)",
 		NULL,
 		&scroll_less
 	},
@@ -607,11 +688,20 @@ static const opt_desc_t options[] =
 		0,
 		OPT_INTEGER,
 		"v",
-		"Amp. of scrolling (% of screen size)",
+		"Amplitude of scrolling (% of screen size)",
 		NULL,
 		&scroll_more
 	},
 
+	{	"sector_render_default",
+		0,
+		OPT_INTEGER,
+		"v",
+		"Default sector rendering mode: 0 = NONE, 1 = floor, 2 = ceiling",
+		NULL,
+		&sector_render_default
+	},
+
 	{	"show_full_one_sided",
 		0,
 		OPT_BOOLEAN,
@@ -630,20 +720,6 @@ static const opt_desc_t options[] =
 		&swap_sidedefs
 	},
 
-// -AJA- this may be better done with a 'mouse_button_order' string,
-//       where the default is "123", and user could have "321"
-//       or "132" or any other combination.
-#if 0
-	{	"swap_buttons",
-		0,
-		OPT_BOOLEAN,
-		"v",
-		"Swap mouse buttons",
-		NULL,
-		&swap_buttons
-	},
-#endif
-
 	//
 	// That's all there is
 	//
