@@ -331,7 +331,10 @@ static void ParseMiscConfig(FILE * fp)
 		}
 		else if (strcmp(line, "known_iwad") == 0)
 		{
-			if (Wad_file::Validate(pos))
+			// ignore plain freedoom.wad (backwards compatibility)
+			if (y_stricmp(map, "freedoom") == 0)
+				LogPrintf("  ignoring for compatibility: %s\n", pos);
+			else if (Wad_file::Validate(pos))
 				known_iwads[map] = std::string(pos);
 			else
 				LogPrintf("  no longer exists: %s\n", pos);
@@ -707,9 +710,16 @@ const char * M_PickDefaultIWAD()
 	if (result)
 		return result;
 
-	DebugPrintf("pick default iwad, trying: 'freedoom'\n");
+	// try FreeDoom
 
-	result = StringDup(M_QueryKnownIWAD("freedoom"));
+	if (strcmp(default_game, "doom") == 0)
+		default_game = "freedoom1";
+	else
+		default_game = "freedoom2";
+
+	DebugPrintf("pick default iwad, trying: '%s'\n", default_game);
+
+	result = StringDup(M_QueryKnownIWAD(default_game));
 	if (result)
 		return result;
 
@@ -920,7 +930,12 @@ void M_WriteEurekaLump(Wad_file *wad)
 		lump->Printf("port %s\n", Port_name);
 
 	for (unsigned int i = 0 ; i < Resource_list.size() ; i++)
-		lump->Printf("resource %s\n", Resource_list[i]);
+	{
+		char absolute_name[FL_PATH_MAX];
+		fl_filename_absolute(absolute_name, Resource_list[i]);
+
+		lump->Printf("resource %s\n", absolute_name);
+	}
 
 	lump->Finish();
 
