@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2014 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -54,6 +54,11 @@ std::map<std::string, char> flat_assigns;
 game_info_t  game_info;  // contains sky_color (etc)
 
 
+generalized_linetype_t gen_linetypes[MAX_GEN_NUM_TYPES];
+
+int num_gen_linetypes;
+
+
 /*
  *  Create empty lists for game definitions
  */
@@ -79,6 +84,10 @@ void M_InitDefinitions()
 	game_info.player_h = 56;
 	game_info.min_dm_starts = 4;
 	game_info.max_dm_starts = 10;
+
+	// reset generalized types
+	memset(&gen_linetypes, 0, sizeof(gen_linetypes));
+	num_gen_linetypes = 0;
 }
 
 
@@ -91,6 +100,7 @@ static short ParseThingdefFlags(const char *s)
 	if (strchr(s, 'l')) flags |= THINGDEF_LIT;
 	if (strchr(s, 'n')) flags |= THINGDEF_PASS;
 	if (strchr(s, 'v')) flags |= THINGDEF_VOID;
+	if (strchr(s, 't')) flags |= THINGDEF_TELEPT;
 
 	return flags;
 }
@@ -162,6 +172,18 @@ static void ParseFeatureDef(char ** argv, int argc)
 	{
 		game_info.friend_flag = atoi(argv[1]);
 	}
+	else if (y_stricmp(argv[0], "pass_through") == 0)
+	{
+		game_info.pass_through = atoi(argv[1]);
+	}
+	else if (y_stricmp(argv[0], "3d_midtex") == 0)
+	{
+		game_info.midtex_3d = atoi(argv[1]);
+	}
+	else if (y_stricmp(argv[0], "medusa_bug") == 0)
+	{
+		game_info.medusa_bug = atoi(argv[1]);
+	}
 	else
 	{
 		LogPrintf("unknown feature keyword: '%s'\n", argv[0]);
@@ -258,14 +280,17 @@ void M_ParseDefinitionFile(const char *filename, const char *folder,
 		basename = fl_filename_name(filename);
 
 
-#define YGD_BUF 200   /* max. line length + 2 */
+#define YGD_BUF  512   /* max. line length + 2 */
 	char readbuf[YGD_BUF];    /* buffer the line is read into */
 
-#define MAX_TOKENS 10   /* tokens per line */
+#define MAX_TOKENS  30   /* tokens per line */
 	int lineno;     /* current line of file */
 
 #define MAX_INCLUDE_LEVEL  10
 
+	int current_gen_line = -1;
+
+
 
 	FILE *fp = fopen(filename, "r");
 	if (! fp)
@@ -433,17 +458,30 @@ void M_ParseDefinitionFile(const char *filename, const char *folder,
 			line_groups[lg->group] = lg;
 		}
 
-		else if (y_stricmp(token[0], "line") == 0)
+		else if (y_stricmp(token[0], "line") == 0 ||
+				 y_stricmp(token[0], "special") == 0)
 		{
+			if (nargs < 3)
+				FatalError(bad_arg_count, basename, lineno, token[0], 3);
+
 			linetype_t * info = new linetype_t;
 
-			if (nargs != 3)
-				FatalError(bad_arg_count, basename, lineno, token[0], 3);
+			memset(info->args, 0, sizeof(info->args));
 
 			int number = atoi(token[1]);
 
-			 info->group = token[2][0];
-			 info->desc  = token[3];
+			info->group = token[2][0];
+			info->desc  = token[3];
+
+			int arg_count = MIN(nargs - 3, 5);
+
+			for (int i = 0 ; i < arg_count ; i++)
+			{
+				if (token[4 + i][0] != '-')
+					info->args[i] = token[4 + i];
+			}
+
+			// FIXME : have separate tables for "special"
 
 			if (line_groups.find( info->group) == line_groups.end())
 			{
@@ -557,6 +595,79 @@ void M_ParseDefinitionFile(const char *filename, const char *folder,
 				flat_assigns[name] = group;
 		}
 
+		else if (y_stricmp(token[0], "gen_line") == 0)
+		{
+			if (nargs != 4)
+				FatalError(bad_arg_count, basename, lineno, token[0], 4);
+
+			current_gen_line = num_gen_linetypes;
+			num_gen_linetypes++;
+
+			if (num_gen_linetypes > MAX_GEN_NUM_TYPES)
+				FatalError("%s(%d): too many gen_line definitions\n", basename, lineno);
+
+			generalized_linetype_t *def = &gen_linetypes[current_gen_line];
+
+			def->key = token[1][0];
+	
+			// use strtol() to support "0x" notation
+			def->base   = strtol(token[2], NULL, 0);
+			def->length = strtol(token[3], NULL, 0);
+
+			def->name = token[4];
+			def->num_fields = 0;
+		}
+
+		else if (y_stricmp(token[0], "gen_field") == 0)
+		{
+			if (nargs < 5)
+				FatalError(bad_arg_count, basename, lineno, token[0], 5);
+
+			if (current_gen_line < 0)
+				FatalError("%s(%d): gen_field used outside of a gen_line definition\n", basename, lineno);
+			
+			generalized_linetype_t *def = &gen_linetypes[current_gen_line];
+
+			generalized_field_t *field = &def->fields[def->num_fields];
+
+			def->num_fields++;
+			if (def->num_fields > MAX_GEN_NUM_FIELDS)
+				FatalError("%s(%d): too many fields in gen_line definition\n", basename, lineno);
+
+			field->bits  = atoi(token[1]);
+			field->shift = atoi(token[2]);
+
+			field->mask  = ((1 << field->bits) - 1) << field->shift;
+
+			field->default_val = atoi(token[3]);
+
+			field->name = token[4];
+
+			// grab the keywords
+			field->num_keywords = MIN(nargs - 4, MAX_GEN_FIELD_KEYWORDS);
+
+			for (int i = 0 ; i < field->num_keywords ; i++)
+			{
+				field->keywords[i] = token[5 + i]; 
+			}
+		}
+
+		else if (y_stricmp(token[0], "exclude_game") == 0)
+		{
+			if (nargs != 1)
+				FatalError(bad_arg_count, basename, lineno, token[0], 1);
+
+			if (Game_name && y_stricmp(token[1], Game_name) == 0)
+			{
+				LogPrintf("WARNING: skipping %s -- not compatible with %s\n",
+						  basename, Game_name);
+
+				// do not process any more of the file
+				fclose(fp);
+				return;
+			}
+		}
+
 		else
 		{
 			FatalError("%s(%d): unknown directive: %.32s\n",
@@ -767,6 +878,60 @@ char M_GetFlatType(const char *name)
 }
 
 
+static bool LineCategory_IsUsed(char group)
+{
+	std::map<int, linetype_t *>::iterator IT;
+
+	for (IT = line_types.begin() ; IT != line_types.end() ; IT++)
+	{
+		linetype_t *info = IT->second;
+		if (info->group == group)
+			return true;
+	}
+
+	return false;
+}
+
+
+static bool ThingCategory_IsUsed(char group)
+{
+	std::map<int, thingtype_t *>::iterator IT;
+
+	for (IT = thing_types.begin() ; IT != thing_types.end() ; IT++)
+	{
+		thingtype_t *info = IT->second;
+		if (info->group == group)
+			return true;
+	}
+
+	return false;
+}
+
+
+static bool TextureCategory_IsUsed(char group)
+{
+	std::map<std::string, char>::iterator IT;
+
+	for (IT = texture_assigns.begin() ; IT != texture_assigns.end() ; IT++)
+		if (IT->second == group)
+			return true;
+
+	return false;
+}
+
+
+static bool FlatCategory_IsUsed(char group)
+{
+	std::map<std::string, char>::iterator IT;
+
+	for (IT = flat_assigns.begin() ; IT != flat_assigns.end() ; IT++)
+		if (IT->second == group)
+			return true;
+
+	return false;
+}
+
+
 const char *M_LineCategoryString(char *letters)
 {
 	static char buffer[2000];
@@ -787,6 +952,9 @@ const char *M_LineCategoryString(char *letters)
 		if (G->group == '-')
 			continue;
 
+		if (! LineCategory_IsUsed(G->group))
+			continue;
+
 		// FIXME: potential for buffer overflow here
 		strcat(buffer, "|");
 		strcat(buffer, G->desc);
@@ -824,6 +992,9 @@ const char *M_ThingCategoryString(char *letters)
 		if (G->group == '-')
 			continue;
 
+		if (! ThingCategory_IsUsed(G->group))
+			continue;
+
 		// FIXME: potential for buffer overflow here
 		strcat(buffer, "|");
 		strcat(buffer, G->desc);
@@ -840,7 +1011,7 @@ const char *M_ThingCategoryString(char *letters)
 }
 
 
-const char *M_TextureCategoryString(char *letters)
+const char *M_TextureCategoryString(char *letters, bool do_flats)
 {
 	static char buffer[2000];
 
@@ -861,6 +1032,12 @@ const char *M_TextureCategoryString(char *letters)
 		if (G->group == '-')
 			continue;
 
+		if (do_flats && !FlatCategory_IsUsed(G->group))
+			continue;
+
+		if (!do_flats && !TextureCategory_IsUsed(G->group))
+			continue;
+
 		// FIXME: potential for buffer overflow here
 		strcat(buffer, "|");
 		strcat(buffer, G->desc);
