@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -101,7 +101,7 @@ bool auto_load_recent = false;
 bool begin_maximized  = false;
 bool map_scroll_bars  = true;
 
-#define DEFAULT_PORT_NAME  "boom"
+#define DEFAULT_PORT_NAME  "vanilla"
 
 const char *default_port = DEFAULT_PORT_NAME;
 
@@ -626,15 +626,7 @@ static void Main_OpenWindow()
 
 	main_win->ShowBrowser(0);
 
-	switch (edit.mode)
-	{
-		case OBJ_LINEDEFS: main_win->NewEditMode('l'); break;
-		case OBJ_SECTORS:  main_win->NewEditMode('s'); break;
-		case OBJ_VERTICES: main_win->NewEditMode('v'); break;
-		case OBJ_THINGS:   main_win->NewEditMode('t'); break;
-
-		default: break;
-	}
+	main_win->NewEditMode(edit.mode);
 
 	Fl::check();
 }
@@ -683,16 +675,6 @@ void Main_Loop()
     {
         Fl::wait(0.2);
 
-        if (edit.RedrawMap)
-        {
-			if (edit.render3d)
-				main_win->render->redraw();
-			else
-				main_win->canvas->redraw();
-
-            edit.RedrawMap = 0;
-        }
-
 		if (want_quit)
 		{
 			if (Main_ConfirmQuit("quit"))
@@ -819,7 +801,7 @@ void Main_LoadResources()
 static void ShowHelp()
 {
 	printf(	"\n"
-			"*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2015 Andrew Apted, et al ***\n"
+			"*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2016 Andrew Apted, et al ***\n"
 			"\n");
 
 	printf(	"Eureka is free software, under the terms of the GNU General\n"
@@ -909,7 +891,7 @@ int main(int argc, char *argv[])
 	init_progress = 1;
 
 	LogPrintf("\n");
-	LogPrintf("*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2015 Andrew Apted, et al ***\n");
+	LogPrintf("*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2016 Andrew Apted, et al ***\n");
 	LogPrintf("\n");
 
 	// Sanity checks (useful when porting).
