@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -46,30 +46,8 @@
 // config items
 bool new_islands_are_void = false;
 int  new_sector_size = 128;
-bool reselect_second_vertex = false;
-
-
-/*
-   get the number of objects of a given type minus one
-*/
-int GetMaxObjectNum (int objtype)
-{
-	switch (objtype)
-	{
-		case OBJ_THINGS:
-			return NumThings - 1;
-		case OBJ_LINEDEFS:
-			return NumLineDefs - 1;
-		case OBJ_SIDEDEFS:
-			return NumSideDefs - 1;
-		case OBJ_VERTICES:
-			return NumVertices - 1;
-		case OBJ_SECTORS:
-			return NumSectors - 1;
-	}
-	return -1;
-}
 
+bool easier_drawing_mode = true;
 
 
 /*
@@ -139,8 +117,8 @@ static void CreateSquare(int model)
 	}
 
 	// select it
-	edit.error_mode = false;
-	edit.Selected->clear_all();
+	Selection_Clear();
+
 	edit.Selected->set(new_sec);
 }
 
@@ -167,12 +145,14 @@ static void Insert_Thing()
 	T->x = grid.SnapX(edit.map_x);
 	T->y = grid.SnapY(edit.map_y);
 
+	recent_things.insert_number(T->type);
+
 	BA_End();
 
 
 	// select it
-	edit.error_mode = false;
-	edit.Selected->clear_all();
+	Selection_Clear();
+
 	edit.Selected->set(new_t);
 }
 
@@ -420,8 +400,18 @@ if ( left_ok) DebugPrintf(" left faces outward : %s\n",  left_loop.faces_outward
 
 		Sectors[new_sec]->RawCopy(Sectors[right_front]);
 
-		AssignSectorToLoop( mod_loop, new_sec,     flip);
-		AssignSectorToLoop(keep_loop, right_front, flip);
+		// ensure the new linedef usually ends at v2 (the final vertex)
+		if (left_total < right_total)
+		{
+			AssignSectorToLoop(keep_loop, right_front, flip);
+			AssignSectorToLoop( mod_loop, new_sec,     flip);
+		}
+		else
+		{
+			AssignSectorToLoop( mod_loop, new_sec,     flip);
+			AssignSectorToLoop(keep_loop, right_front, flip);
+		}
+
 		return;
 	}
 
@@ -508,7 +498,7 @@ DebugPrintf("ISLANDS = %u\n", loop.islands.size());
 }
 
 
-static void Insert_LineDef(int v1, int v2)
+void Insert_LineDef(int v1, int v2, bool no_fill = false)
 {
 	int new_ld = BA_New(OBJ_LINEDEFS);
 
@@ -518,6 +508,9 @@ static void Insert_LineDef(int v1, int v2)
 	L->end   = v2;
 	L->flags = MLF_Blocking;
 
+	if (no_fill)
+		return;
+
 	selection_c flip(OBJ_LINEDEFS);
 
 	switch (VertexHowManyLineDefs(v2))
@@ -542,20 +535,53 @@ static void Insert_LineDef(int v1, int v2)
 }
 
 
-static void Insert_Vertex()
+void Insert_LineDef_autosplit(int v1, int v2, bool no_fill = false)
 {
-	int reselect = true;
+	if (LineDefAlreadyExists(v1, v2))
+		return;
 
-	if (edit.Selected->count_obj() > 2)
+fprintf(stderr, "Insert_LineDef_autosplit %d..%d\n", v1, v2);
+
+	// Find a linedef which this new line would cross, and if it exists
+	// add a vertex there and create TWO lines.  Also handle a vertex
+	// that this line crosses (sits on) similarly.
+
+	cross_state_t cross;
+
+	if (! FindClosestCrossPoint(v1, v2, &cross))
 	{
-		Beep("Too many vertices to add a linedef");
+		Insert_LineDef(v1, v2, no_fill);
 		return;
 	}
 
+	if (cross.line >= 0)
+	{
+		cross.vert = BA_New(OBJ_VERTICES);
 
-	// the "nearby" vertex is usually the highlighted one (if any),
-	// but we substitute the vertex at the snapped coord when the
-	// highlight is empty and snapping is enabled.
+		Vertex *V = Vertices[cross.vert];
+
+		V->x = cross.x;
+		V->y = cross.y;
+
+		SplitLineDefAtVertex(cross.line, cross.vert);
+	}
+
+	// recursively handle both sides
+	
+	Insert_LineDef_autosplit(v1, cross.vert, no_fill);
+	Insert_LineDef_autosplit(cross.vert, v2, no_fill);
+}
+
+
+void Insert_Vertex(bool force_continue, bool no_fill, bool is_button)
+{
+	bool do_continue = true;
+
+	int from_vert = -1;
+	int   to_vert = -1;
+
+
+	// the "nearby" vertex is usually the highlighted one.
 	int near_vert = -1;
 
 	if (edit.highlight.valid())
@@ -564,145 +590,187 @@ static void Insert_Vertex()
 	int new_x = grid.SnapX(edit.map_x);
 	int new_y = grid.SnapY(edit.map_y);
 
-	if (near_vert < 0 && grid.snap)
+
+	// a linedef which we are splitting (usually none)
+	int split_ld = edit.split_line.valid() ? edit.split_line.num : -1;
+
+
+	if (easier_drawing_mode)
+	{
+		if (edit.action == ACT_DRAW_LINE)
+			from_vert = edit.drawing_from;
+	}
+	else
+	{
+		if (edit.Selected->count_obj() > 2)
+		{
+			Beep("Too many vertices to add a linedef");
+			return;
+		}
+
+		from_vert = edit.Selected->find_first();
+		  to_vert = edit.Selected->find_second();
+
+		if (from_vert >= 0 && to_vert >= 0 && split_ld >= 0)
+		{
+			Beep("Too many vertices to split a linedef");
+			return;
+		}
+	}
+
+
+	// if no highlight, look for a vertex at snapped coord
+	if (! (edit.action == ACT_DRAW_LINE) && near_vert < 0 && grid.snap)
+	{
 		near_vert = Vertex_FindExact(new_x, new_y);
-
-
-	int first_sel  = edit.Selected->find_first();
-	int second_sel = edit.Selected->find_second();
-
-	// if a vertex is highlighted but none are selected, then merely
-	// select that vertex.  This is better than adding a new vertex at
-	// the same location as an existing one.
-	if (edit.Selected->empty() && near_vert >= 0)
-	{
-		edit.Selected->set(near_vert);
-		return;
 	}
 
-	// if highlighted vertex same as selected one, merely deselect it
-	if (second_sel < 0 && near_vert >= 0 && near_vert == first_sel)
-	{
-		edit.Selected->clear(first_sel);
-		return;
-	}
 
-	// if only one is selected, but another is highlighted, use the
-	// highlighted one as the second vertex
-	if (second_sel < 0 && near_vert >= 0)
+	//
+	// handle a highlighted vertex
+	// [ an explicit destination overrides any highlight ]
+	// [ a splittable line also overrides ]
+	//
+	if (near_vert >= 0 && to_vert < 0 && split_ld < 0)
 	{
-		second_sel = near_vert;
-	}
-
-	// insert a new linedef between two existing vertices
-	if (first_sel >= 0 && second_sel >= 0)
-	{
-		if (LineDefAlreadyExists(first_sel, second_sel))
+		// the simple "select it" case, as we have no explicit source
+		if (from_vert < 0)
 		{
-			edit.Selected->clear(first_sel);
-			edit.Selected->set  (second_sel);
+			// a plain INSERT will attempt to fix a dangling vertex
+			if (!is_button && edit.action == ACT_NOTHING)
+			{
+				if (Vertex_TryFixDangler(near_vert))
+				{
+					// a vertex was deleted, selection/highlight is now invalid
+					return;
+				}
+			}
+
+			edit.Selected->set(near_vert);
+
+			if (easier_drawing_mode)
+			{
+				Editor_SetAction(ACT_DRAW_LINE);
+				edit.drawing_from = near_vert;
+			}
+
 			return;
 		}
 
-		if (LineDefWouldOverlap(first_sel, Vertices[second_sel]->x, Vertices[second_sel]->y) ||
-		    Vertices[first_sel]->Matches(Vertices[second_sel]))
+		
+		// the simple "unselect it" case
+		if (near_vert == from_vert)
 		{
-			Beep("New linedef would overlap another");
+			edit.Selected->clear(from_vert);
+			Editor_ClearAction();
 			return;
 		}
 
-	    // TODO: CONFIG ITEM to always reselect second
-		if (!reselect_second_vertex && VertexHowManyLineDefs(second_sel) > 0)
-			reselect = false;
+
+		// we have no explicit destination, so use the highlight
+		// [ in drawing mode we never have an explicit destination ]
+
+		to_vert = near_vert;
+
+
+		// 'near_vert' is no longer used...
+	}
+
+
+	if (from_vert >= 0 && to_vert >= 0)
+	{
+		/* ------ adding a linedef, no new vertex ------ */
+
+		if (LineDefAlreadyExists(from_vert, to_vert))
+		{
+			edit.Selected->set(from_vert);
+			edit.Selected->set(to_vert);
+
+			Editor_ClearAction();
+			return;
+		}
+
+		if (!force_continue && VertexHowManyLineDefs(to_vert) > 0)
+			do_continue = false;
 
 		BA_Begin();
 
-		Insert_LineDef(first_sel, second_sel);
+		Insert_LineDef_autosplit(from_vert, to_vert, no_fill);
 
 		BA_End();
-
-		edit.Selected->clear_all();
-		if (reselect)
-			edit.Selected->set(second_sel);
-
-		return;
 	}
-
-	// make sure we never create zero-length linedefs
-
-	if (first_sel >= 0)
+	else
 	{
-		Vertex *V = Vertices[first_sel];
+		/* ------ creating a new vertex ------ */
 
-		if (V->Matches(new_x, new_y))
+		// do not create a new line when we are splitting a line and
+		// the source vertex is an endpoint of that line (otherwise
+		// we would get two overlapping lines).
+
+		if (from_vert >= 0 && split_ld >= 0 &&
+			LineDefs[split_ld]->TouchesVertex(from_vert))
 		{
-			edit.Selected->clear_all();
+			from_vert = -1;
+		}
+
+
+		// the following should not happen, but just in case...
+		if (from_vert >= 0 && split_ld < 0 &&
+			Vertices[from_vert]->x == new_x &&
+			Vertices[from_vert]->y == new_y)
+		{
+			Beep("Bug detected (creation of zero-length line)");
 			return;
 		}
-	}
 
 
-	// handle case of splitting a line where the first selected vertex
-	// matches an endpoint of that line -- we just want to split the
-	// line then (NOT insert a new linedef)
-	if (first_sel >= 0 && edit.split_line.valid() &&
-	    (first_sel == LineDefs[edit.split_line.num]->start ||
-		 first_sel == LineDefs[edit.split_line.num]->end))
-	{
-		first_sel = -1;
-	}
+		BA_Begin();
 
+		to_vert = BA_New(OBJ_VERTICES);
 
-	// prevent adding a linedef that would overlap an existing one
-	if (first_sel >= 0 && LineDefWouldOverlap(first_sel, new_x, new_y))
-	{
-		Beep("New linedef would overlap another");
-		return;
-	}
+		Vertex *V = Vertices[to_vert];
 
+		V->x = new_x;
+		V->y = new_y;
 
-	BA_Begin();
-
-	int new_v = BA_New(OBJ_VERTICES);
-
-	Vertex *V = Vertices[new_v];
-
-	V->x = new_x;
-	V->y = new_y;
-
-	// split an existing linedef?
-	if (edit.split_line.valid())
-	{
-		// in FREE mode, ensure the new vertex is directly on the linedef
-		if (! grid.snap)
+		// split an existing linedef?
+		if (split_ld >= 0)
 		{
-			MoveCoordOntoLineDef(edit.split_line.num, &new_x, &new_y);
+			V->x = edit.split_x;
+			V->y = edit.split_y;
 
-			V->x = new_x;
-			V->y = new_y;
+			SplitLineDefAtVertex(split_ld, to_vert);
+
+			if (!force_continue && from_vert >= 0)
+				do_continue = false;
 		}
 
-		SplitLineDefAtVertex(edit.split_line.num, new_v);
+		// add a new linedef?
+		if (from_vert >= 0)
+		{
+			Insert_LineDef_autosplit(from_vert, to_vert, no_fill);
+		}
 
-		// CONFIG ITEM
-		if (!reselect_second_vertex && first_sel >= 0)
-			reselect = false;
+		BA_End();
 	}
 
-	// add a new linedef?
-	if (first_sel >= 0)
+
+	Selection_Clear();
+	Editor_ClearAction();
+
+	// continue drawing / select new vertex
+	if (do_continue)
 	{
-		Insert_LineDef(first_sel, new_v);
+		edit.Selected->set(to_vert);
+
+		if (easier_drawing_mode)
+		{
+			Editor_SetAction(ACT_DRAW_LINE);
+			edit.drawing_from = to_vert;
+		}
 	}
 
-	BA_End();
-
-	// select new vertex
-	edit.error_mode = false;
-	edit.Selected->clear_all();
-
-	if (reselect)
-		edit.Selected->set(new_v);
+	RedrawMap();
 }
 
 
@@ -738,8 +806,6 @@ static void Insert_Sector(bool force_new)
 
 		BA_End();
 
-		edit.Selected->clear_all();
-		edit.Selected->set(NumSectors - 1);
 		return;
 	}
 
@@ -784,14 +850,42 @@ static void Insert_Sector(bool force_new)
 	BA_End();
 
 
-	edit.error_mode = false;
-	edit.Selected->clear_all();
+	Selection_Clear();
+
 	edit.Selected->set(new_sec);
 }
 
 
+void Insert_Vertex_split(int split_ld, int new_x, int new_y)
+{
+	BA_Begin();
+
+	int new_vert = BA_New(OBJ_VERTICES);
+
+	Vertex *V = Vertices[new_vert];
+
+	V->x = new_x;
+	V->y = new_y;
+
+	SplitLineDefAtVertex(split_ld, new_vert);
+
+	BA_End();
+
+	Selection_Clear();
+	Editor_ClearAction();
+
+	edit.clicked = Objid(OBJ_VERTICES, new_vert);
+
+	RedrawMap();
+}
+
+
 void CMD_Insert(void)
 {
+	bool force_new;
+	bool force_cont;
+	bool no_fill;
+
 	switch (edit.mode)
 	{
 		case OBJ_THINGS:
@@ -799,15 +893,14 @@ void CMD_Insert(void)
 			break;
 
 		case OBJ_VERTICES:
-			Insert_Vertex();
+			force_cont = Exec_HasFlag("/continue");
+			no_fill    = Exec_HasFlag("/nofill");
+			Insert_Vertex(force_cont, no_fill);
 			break;
 
 		case OBJ_SECTORS:
-			{
-				bool force_new = Exec_HasFlag("/new");
-
-				Insert_Sector(force_new);
-			}
+			force_new = Exec_HasFlag("/new");
+			Insert_Sector(force_new);
 			break;
 
 		default:
@@ -815,7 +908,7 @@ void CMD_Insert(void)
 			break;
 	}
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -973,12 +1066,10 @@ success:
 }
 
 
-void TransferThingProperties(int src_thing, int dest_thing)
+static void TransferThingProperties(int src_thing, int dest_thing)
 {
 	const Thing * T = Things[src_thing];
 
-	BA_Begin();
-
 	BA_ChangeTH(dest_thing, Thing::F_TYPE,    T->type);
 	BA_ChangeTH(dest_thing, Thing::F_OPTIONS, T->options);
 //	BA_ChangeTH(dest_thing, Thing::F_ANGLE,   T->angle);
@@ -991,17 +1082,13 @@ void TransferThingProperties(int src_thing, int dest_thing)
 	BA_ChangeTH(dest_thing, Thing::F_ARG3, T->arg3);
 	BA_ChangeTH(dest_thing, Thing::F_ARG4, T->arg4);
 	BA_ChangeTH(dest_thing, Thing::F_ARG5, T->arg5);
-
-	BA_End();
 }
 
 
-void TransferSectorProperties(int src_sec, int dest_sec)
+static void TransferSectorProperties(int src_sec, int dest_sec)
 {
 	const Sector * SEC = Sectors[src_sec];
 
-	BA_Begin();
-
 	BA_ChangeSEC(dest_sec, Sector::F_FLOORH,    SEC->floorh);
 	BA_ChangeSEC(dest_sec, Sector::F_FLOOR_TEX, SEC->floor_tex);
 	BA_ChangeSEC(dest_sec, Sector::F_CEILH,     SEC->ceilh);
@@ -1010,14 +1097,12 @@ void TransferSectorProperties(int src_sec, int dest_sec)
 	BA_ChangeSEC(dest_sec, Sector::F_LIGHT,  SEC->light);
 	BA_ChangeSEC(dest_sec, Sector::F_TYPE,   SEC->type);
 	BA_ChangeSEC(dest_sec, Sector::F_TAG,    SEC->tag);
-
-	BA_End();
 }
 
 
 #define LINEDEF_FLAG_KEEP  (MLF_Blocking + MLF_TwoSided)
 
-void TransferLinedefProperties(int src_line, int dest_line, bool do_tex)
+static void TransferLinedefProperties(int src_line, int dest_line, bool do_tex)
 {
 	const LineDef * L1 = LineDefs[src_line];
 	const LineDef * L2 = LineDefs[dest_line];
@@ -1026,8 +1111,6 @@ void TransferLinedefProperties(int src_line, int dest_line, bool do_tex)
 	int flags = LineDefs[dest_line]->flags;
 	flags = (flags & LINEDEF_FLAG_KEEP) | (L1->flags & ~LINEDEF_FLAG_KEEP);
 
-	BA_Begin();
-
 	// handle textures
 	if (do_tex && L1->Right() && L2->Right())
 	{
@@ -1154,8 +1237,6 @@ void TransferLinedefProperties(int src_line, int dest_line, bool do_tex)
 	BA_ChangeLD(dest_line, LineDef::F_ARG3, L1->arg3);
 	BA_ChangeLD(dest_line, LineDef::F_ARG4, L1->arg4);
 	BA_ChangeLD(dest_line, LineDef::F_ARG5, L1->arg5);
-
-	BA_End();
 }
 
 
@@ -1171,38 +1252,90 @@ void CMD_CopyProperties(void)
 		Beep("No source for CopyProperties");
 		return;
 	}
-	else if (edit.Selected->count_obj() != 1)
+	else if (edit.mode == OBJ_VERTICES)
 	{
-		Beep("Too many sources for CopyProperties");
+		Beep("No properties to copy");
 		return;
 	}
 
 
-	int source = edit.Selected->find_first();
-	int target = edit.highlight.num;
+	/* normal mode, SEL --> HILITE */
 
-	// silently allow copying onto self
-	if (source == target)
-		return;
-
-
-	switch (edit.mode)
+	if (! Exec_HasFlag("/reverse"))
 	{
-		case OBJ_SECTORS:
-			TransferSectorProperties(source, target);
-			break;
-
-		case OBJ_THINGS:
-			TransferThingProperties(source, target);
-			break;
-
-		case OBJ_LINEDEFS:
-			TransferLinedefProperties(source, target, true /* do_tex */);
-			break;
-
-		default:
-			Beep("No properties to copy");
+		if (edit.Selected->count_obj() != 1)
+		{
+			Beep("Too many sources for CopyProperties");
 			return;
+		}
+
+		int source = edit.Selected->find_first();
+		int target = edit.highlight.num;
+
+		// silently allow copying onto self
+		if (source == target)
+			return;
+
+		BA_Begin();
+
+		switch (edit.mode)
+		{
+			case OBJ_SECTORS:
+				TransferSectorProperties(source, target);
+				break;
+
+			case OBJ_THINGS:
+				TransferThingProperties(source, target);
+				break;
+
+			case OBJ_LINEDEFS:
+				TransferLinedefProperties(source, target, true /* do_tex */);
+				break;
+
+			default: break;
+		}
+
+		BA_End();
+
+	}
+	else  /* reverse mode, HILITE --> SEL */
+	{
+		if (edit.Selected->count_obj() == 1 && edit.Selected->find_first() == edit.highlight.num)
+		{
+			Beep("No selection for CopyProperties");
+			return;
+		}
+
+		int source = edit.highlight.num;
+
+		selection_iterator_c it;
+
+		BA_Begin();
+
+		for (edit.Selected->begin(&it) ; !it.at_end() ; ++it)
+		{
+			if (*it == source)
+				continue;
+
+			switch (edit.mode)
+			{
+				case OBJ_SECTORS:
+					TransferSectorProperties(source, *it);
+					break;
+
+				case OBJ_THINGS:
+					TransferThingProperties(source, *it);
+					break;
+
+				case OBJ_LINEDEFS:
+					TransferLinedefProperties(source, *it, true /* do_tex */);
+					break;
+
+				default: break;  // fuck you, compiler
+			}
+		}
+
+		BA_End();
 	}
 }
 
