@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -52,7 +52,7 @@
 
 
 // config items
-int  render_aspect_ratio = 133;   // 100 * width / height
+int  render_pixel_aspect = 100;   // 100 * width / height
 
 bool render_high_detail    = false;
 bool render_lock_gravity   = false;
@@ -127,10 +127,6 @@ public:
 	int thsec_sector_num;
 	bool thsec_invalidated;
 
-	Img_c *missing_tex;  int missing_col;
-	Img_c *unknown_tex;  int unk_tex_col;
-	Img_c *unknown_flat; int unk_flat_col;
-
 	// state for adjusting offsets via the mouse
 	int   adjust_ld;
 	int   adjust_sd;
@@ -146,10 +142,8 @@ public:
 			   texturing(false), sprites(false), lighting(false),
 			   gravity(true),
 	           thing_sectors(),
-			   thsec_sector_num(0), thsec_invalidated(false),
-			   missing_tex(NULL),  missing_col(-1),
-			   unknown_tex(NULL),  unk_tex_col(-1),
-			   unknown_flat(NULL), unk_flat_col(-1),
+			   thsec_sector_num(0),
+			   thsec_invalidated(false),
 			   adjust_ld(-1), adjust_sd(-1),
 			   hl()
 	{ }
@@ -170,7 +164,7 @@ public:
 	void CalcViewZ()
 	{
 		Objid o;
-		GetCurObject(o, OBJ_SECTORS, int(x), int(y));
+		GetNearObject(o, OBJ_SECTORS, int(x), int(y));
 
 		int secnum = o.num;
 
@@ -180,10 +174,11 @@ public:
 
 	void CalcAspect()
 	{
-		float screen_aspect = float(sw) / float(sh);
+		float window_aspect = float(sw) / float(sh);
 
-		aspect_sh = sh / (render_aspect_ratio / 100.0) * screen_aspect;
-		aspect_sw = sw;
+		aspect_sh = sh * window_aspect * (render_pixel_aspect / 133.0);
+
+		aspect_sw = sw;	 // things break if these are different
 	}
 
 	void UpdateScreen(int ow, int oh)
@@ -222,7 +217,7 @@ public:
 		{
 			Objid obj;
 
-			GetCurObject(obj, OBJ_SECTORS, Things[i]->x, Things[i]->y);
+			GetNearObject(obj, OBJ_SECTORS, Things[i]->x, Things[i]->y);
 
 			thing_sectors[i] = obj.num;
 		}
@@ -249,30 +244,6 @@ public:
 		return raw_colormap[map][pixel];
 	}
 
-	void UpdateDummies()
-	{
-		if (missing_col != game_info.missing_color)
-		{
-			missing_col = game_info.missing_color;
-			if (missing_tex) delete missing_tex;
-			missing_tex = IM_CreateMissingTex(missing_col, 0);
-		}
-
-		if (unk_tex_col != game_info.unknown_tex)
-		{
-			unk_tex_col = game_info.unknown_tex;
-			if (unknown_tex) delete unknown_tex;
-			unknown_tex = IM_CreateUnknownTex(unk_tex_col, 0);
-		}
-
-		if (unk_flat_col != game_info.unknown_flat)
-		{
-			unk_flat_col = game_info.unknown_flat;
-			if (unknown_flat) delete unknown_flat;
-			unknown_flat = IM_CreateUnknownTex(unk_flat_col, 0);
-		}
-	}
-
 	void PrepareToRender(int ow, int oh)
 	{
 		if (thsec_invalidated || !screen ||
@@ -282,8 +253,6 @@ public:
 			FindThingSectors();
 		}
 
-		UpdateDummies();
-
 		UpdateScreen(ow, oh);
 
 		if (gravity)
@@ -382,7 +351,7 @@ public:
 
 			if (! img)
 			{
-				img = view.unknown_flat;
+				img = IM_UnknownFlat();
 				fullbright = render_unknown_bright;
 			}
 
@@ -400,7 +369,7 @@ public:
 		{
 			if (tname[0] == '-')
 			{
-				img = view.missing_tex;
+				img = IM_MissingTex();
 				fullbright = render_missing_bright;
 				return;
 			}
@@ -409,7 +378,7 @@ public:
 
 			if (! img)
 			{
-				img = view.unknown_tex;
+				img = IM_UnknownTex();
 				fullbright = render_unknown_bright;
 			}
 
@@ -442,8 +411,8 @@ public:
 	int wall_light;
 
 	// clipped angles
-	float ang1, dang, cur_ang;
-	float base_ang;
+	float ang1;
+	float delta_ang;
 
 	// line constants
 	float dist, t_dist;
@@ -1050,9 +1019,8 @@ public:
 		else if (ld->Start()->y == ld->End()->y)
 			dw->wall_light -= 16;
 
-		dw->base_ang = base_ang;
 		dw->ang1 = angle1;
-		dw->dang = (angle2 - angle1) / MAX(1, sx2 - sx1);
+		dw->delta_ang = angle1 + XToAngle(sx1) - normal;
 
 		dw->dist = dist;
 		dw->normal = normal;
@@ -1112,12 +1080,13 @@ public:
 
 		if (info && (info->flags & THINGDEF_CEIL))
 		{
-			h2 = is_sector(thsec) ? Sectors[thsec]->ceilh : 192;
+			// IOANCH 9/2015: also add z
+			h2 = (is_sector(thsec) ? Sectors[thsec]->ceilh : 192) - th->z;
 			h1 = h2 - sprite->height();
 		}
 		else
 		{
-			h1 = is_sector(thsec) ? Sectors[thsec]->floorh : 0;
+			h1 = (is_sector(thsec) ? Sectors[thsec]->floorh : 0) + th->z;
 			h2 = h1 + sprite->height();
 		}
 
@@ -1134,7 +1103,7 @@ public:
 
 		dw->spr_tx1 = tx1;
 
-		dw->ang1 = dw->dang = 0;
+		dw->ang1 = 0;
 
 		dw->iz1 = dw->mid_iz = iz;
 		dw->diz = 0;
@@ -1303,7 +1272,9 @@ public:
 
 		/* compute texture X coord */
 
-		int tx = int(dw->t_dist - tan(dw->cur_ang - dw->normal) * dw->dist);
+		float cur_ang = dw->delta_ang - XToAngle(x);
+
+		int tx = int(dw->t_dist - tan(cur_ang) * dw->dist);
 
 		tx = (dw->sd->x_offset + tx) & (tw - 1);
 
@@ -1584,8 +1555,6 @@ public:
 
 			if (P != S && (*(P-1))->cur_iz < dw->cur_iz + IZ_EPSILON)
 				changes = true;
-
-			dw->cur_ang = dw->ang1 + dw->dang * (x - dw->sx1);
 		}
 
 		// if there are changes, re-sort the active list...
@@ -2012,7 +1981,7 @@ void Render3D_Setup()
 }
 
 
-void Render3D_MouseMotion(int x, int y, keycode_t mod, bool drag)
+void Render3D_MouseMotion(int x, int y, keycode_t mod)
 {
 	highlight_3D_info_t old(view.hl);
 
@@ -2037,7 +2006,7 @@ void Render3D_Wheel(int dx, int dy, keycode_t mod)
 	view.x += speed * (view.Cos * dy + view.Sin * dx);
 	view.y += speed * (view.Sin * dy - view.Cos * dx);
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -2106,7 +2075,7 @@ void Render3D_RBScroll(int dx, int dy, keycode_t mod)
 		view.gravity = false;
 	}
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -2205,7 +2174,7 @@ void Render3D_AdjustOffsets(int mode, int dx, int dy)
 	view.adjust_dx -= dx * factor * view.adjust_dx_factor;
 	view.adjust_dy -= dy * factor * view.adjust_dy_factor;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -2307,7 +2276,7 @@ void R3D_Forward(void)
 	view.x += view.Cos * dist;
 	view.y += view.Sin * dist;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 void R3D_Backward(void)
@@ -2317,7 +2286,7 @@ void R3D_Backward(void)
 	view.x -= view.Cos * dist;
 	view.y -= view.Sin * dist;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 void R3D_Left(void)
@@ -2327,7 +2296,7 @@ void R3D_Left(void)
 	view.x -= view.Sin * dist;
 	view.y += view.Cos * dist;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 void R3D_Right(void)
@@ -2337,7 +2306,7 @@ void R3D_Right(void)
 	view.x += view.Sin * dist;
 	view.y -= view.Cos * dist;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 void R3D_Up(void)
@@ -2353,7 +2322,7 @@ void R3D_Up(void)
 	view.z += dist;
 	view.gravity = false;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 void R3D_Down(void)
@@ -2369,7 +2338,7 @@ void R3D_Down(void)
 	view.z -= dist;
 	view.gravity = false;
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -2382,14 +2351,14 @@ void R3D_Turn(void)
 
 	view.SetAngle(view.angle + angle);
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 void R3D_DropToFloor(void)
 {
 	view.CalcViewZ();
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -2444,7 +2413,7 @@ void R3D_Set(void)
 		return;
 	}
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -2481,21 +2450,7 @@ void R3D_Toggle(void)
 		return;
 	}
 
-	edit.RedrawMap = 1;
-}
-
-
-void R3D_Gamma(void)
-{
-	int delta = (atoi(EXEC_Param[0]) >= 0) ? +1 : -1;
-
-	usegamma = (usegamma + delta + 5) % 5;
-
-	W_UpdateGamma();
-
-	Status_Set("Gamma level %d", usegamma);
-
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -2613,10 +2568,6 @@ static editor_command_t  render_commands[] =
 		&R3D_DropToFloor
 	},
 
-	{	"3D_Gamma",
-		&R3D_Gamma
-	},
-
 	{	"3D_Set",
 		&R3D_Set,
 		/* flags */ NULL,
