@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2007-2015 Andrew Apted
+//  Copyright (C) 2007-2016 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -60,7 +60,8 @@ typedef enum
 {
 	SOM_Numeric = 0,
 	SOM_Alpha,
-	SOM_AlphaSkip	// skip the S1, WR (etc) of linedef descriptions
+	SOM_AlphaSkip,  // skip the S1, WR (etc) of linedef descriptions
+	SOM_Recent
 
 } sort_method_e;
 
@@ -73,6 +74,7 @@ Browser_Item::Browser_Item(int X, int Y, int W, int H,
 	Fl_Group(X, Y, W, H, ""),
 	desc(_desc), real_name(_realname),
 	number(_num), category(_category),
+	recent_idx(-2),
 	button(NULL), pic(NULL)
 {
 	end();
@@ -97,6 +99,7 @@ Browser_Item::Browser_Item(int X, int Y, int W, int H,
 	Fl_Group(X, Y, W, H, ""),
 	desc(_desc), real_name(_realname),
 	number(_num), category(_category),
+	recent_idx(-2),
 	button(NULL), pic(_pic)
 {
 	end();
@@ -171,7 +174,7 @@ UI_Browser_Box::UI_Browser_Box(int X, int Y, int W, int H, const char *label, ch
 {
 	end(); // cancel begin() in Fl_Group constructor
 
-	box(FL_FLAT_BOX);  // box(FL_DOWN_BOX);
+	box(FL_FLAT_BOX);
 
 	color(BROWBACK_COL, BROWBACK_COL);
 
@@ -274,6 +277,7 @@ UI_Browser_Box::UI_Browser_Box(int X, int Y, int W, int H, const char *label, ch
 
 UI_Browser_Box::~UI_Browser_Box()
 {
+	// nothing needed
 }
 
 
@@ -325,8 +329,6 @@ void UI_Browser_Box::sort_callback(Fl_Widget *w, void *data)
 
 bool UI_Browser_Box::Filter(bool force_update)
 {
-//!!!!	pack->scroll_to(0, 0);
-
 	bool changes = false;
 
 	int left_X  = scroll->x() + SBAR_W;
@@ -401,7 +403,7 @@ bool UI_Browser_Box::SearchMatch(Browser_Item *item) const
 
 		// special logic for RECENT category  [ignore search box]
 		if (cat == '^')
-			return RecentMatch(item);
+			return (item->recent_idx >= 0);
 
 		if (! (cat == tolower(item->category) ||
 			   (cat == 'X' && isupper(item->category))))
@@ -422,22 +424,35 @@ bool UI_Browser_Box::SearchMatch(Browser_Item *item) const
 }
 
 
-bool UI_Browser_Box::RecentMatch(Browser_Item *item) const
+bool UI_Browser_Box::Recent_UpdateItem(Browser_Item *item)
 {
+	// returns true if the index changed
+
+	int new_idx = -1;
+
 	switch (kind)
 	{
 		case 'T':
-			return (recent_textures.find(item->real_name.c_str()) >= 0);
+			new_idx = recent_textures.find(item->real_name.c_str());
+			break;
 
 		case 'F':
-			return (recent_flats.find(item->real_name.c_str()) >= 0);
+			new_idx = recent_flats.find(item->real_name.c_str());
+			break;
 
 		case 'O':
-			return (recent_things.find_number(item->number) >= 0);
+			new_idx = recent_things.find_number(item->number);
+			break;
 
 		default:
 			return false;
 	}
+
+	if (item->recent_idx == new_idx)
+		return false;
+
+	item->recent_idx = new_idx;
+	return true;
 }
 
 
@@ -450,6 +465,10 @@ static int SortCmp(const Browser_Item *A, const Browser_Item *B, sort_method_e m
 	{
 		return (A->number - B->number);
 	}
+	else if (method == SOM_Recent)
+	{
+		return (A->recent_idx - B->recent_idx);
+	}
 
 	if (strchr(sa, '/')) sa = strchr(sa, '/') + 1;
 	if (strchr(sb, '/')) sb = strchr(sb, '/') + 1;
@@ -460,8 +479,11 @@ static int SortCmp(const Browser_Item *A, const Browser_Item *B, sort_method_e m
 		while (isspace(*sa)) sa++;
 		while (isspace(*sb)) sb++;
 
-		while (! isspace(*sa)) sa++;
-		while (! isspace(*sb)) sb++;
+		if (sa[0] && sa[1] && sa[2] == ' ')
+			while (! isspace(*sa)) sa++;
+
+		if (sb[0] && sb[1] && sb[2] == ' ')
+			while (! isspace(*sb)) sb++;
 	}
 
 	return strcmp(sa, sb);
@@ -496,9 +518,13 @@ void UI_Browser_Box::Sort()
 		scroll->Remove_first();
 	}
 
+	char cat = cat_letters[category->value()];
+
 	sort_method_e method = SOM_Alpha;
 
-	if (alpha && ! alpha->value())
+	if (cat == '^')
+		method = SOM_Recent;
+	else if (alpha && ! alpha->value())
 		method = SOM_Numeric;
 	else if (kind == 'L')
 		method = SOM_AlphaSkip;
@@ -792,6 +818,8 @@ void UI_Browser_Box::Populate()
 			break;
 	}
 
+	RecentUpdate();
+
 	// this calls Filter to reposition the widgets
 	Sort();
 }
@@ -799,8 +827,8 @@ void UI_Browser_Box::Populate()
 
 void UI_Browser_Box::SetCategories(const char *cats, const char *letters)
 {
-	// FIXME: possible buffer overflow
-	strcpy(cat_letters, letters);
+	strncpy(cat_letters, letters, sizeof(cat_letters));
+	cat_letters[sizeof(cat_letters) - 1] = 0;
 
 	category->clear();
 	category->add(cats);
@@ -832,7 +860,6 @@ void UI_Browser_Box::CycleCategory(int dir)
 		}
 
 		// skip the RECENT category
-		// TODO : controllable via a /xxx flag
 		if (new_cat != 1)
 			break;
 	}
@@ -882,12 +909,20 @@ void UI_Browser_Box::Scroll(int delta)
 
 void UI_Browser_Box::RecentUpdate()
 {
+	bool changes = false;
+
+	for (int i = 0 ; i < scroll->Children() ; i++)
+	{
+		Browser_Item *item = (Browser_Item *)scroll->Child(i);
+
+		if (Recent_UpdateItem(item))
+			changes = true;
+	}
+
 	char cat = cat_letters[category->value()];
 
-	if (cat == '^')
-	{
-		Filter();
-	}
+	if (changes && cat == '^')
+		Sort();
 }
 
 
@@ -910,7 +945,405 @@ void UI_Browser_Box::ToggleRecent(bool force_recent)
 
 	category->value(new_cat);
 
-	Filter();
+	Sort();
+}
+
+
+//------------------------------------------------------------------------
+
+
+class UI_Generalized_Item : public Fl_Choice
+{
+public:
+	const generalized_field_t * field;
+
+public:
+	UI_Generalized_Item(int X, int Y, int W, int H,
+						const generalized_field_t *_field) :
+		Fl_Choice(X, Y, W, H, ""),
+		field(_field)
+	{
+		char label_buf[256];
+
+		snprintf(label_buf, sizeof(label_buf), "%s: ", field->name);
+
+		copy_label(label_buf);
+
+		for (int i = 0 ; i < field->num_keywords ; i++)
+		{
+			add(field->keywords[i]);
+		}
+
+		Reset();
+	}
+
+	~UI_Generalized_Item()
+	{ }
+
+	int Compute() const
+	{
+		return (value() << field->shift) & field->mask;
+	}
+
+	void Decode(int line_type)
+	{
+		value((line_type & field->mask) >> field->shift);
+	}
+
+	void Reset()
+	{
+		int def_val = CLAMP(0, field->default_val, field->num_keywords - 1);
+
+		value(def_val);
+	}
+};
+
+
+class UI_Generalized_Page : public Fl_Group
+{
+public:
+	int t_base;
+	int t_length;
+
+	UI_Generalized_Item * items[MAX_GEN_NUM_FIELDS];
+	int num_items;
+
+	// index for the "Change", "Model", "Monster" triplet, usually -1
+	int change_index;
+
+	UI_Generalized_Item *change_widget;
+
+private:
+	static void item_callback(Fl_Widget *w, void *data)
+	{
+		UI_Generalized_Page *page = (UI_Generalized_Page *)data;
+
+		if (w == page->change_widget)
+			page->UpdateChange();
+
+		page->do_callback();
+	}
+
+public:
+	UI_Generalized_Page(int X, int Y, int W, int H,
+						const generalized_linetype_t *info) :
+		Fl_Group(X, Y, W, H),
+		t_base(info->base), t_length(info->length),
+		num_items(0),
+		change_index(-1),
+		change_widget(NULL)
+	{
+#if 0
+		box(FL_FLAT_BOX);
+		color(FL_BLUE, FL_BLUE);
+#endif
+
+		memset(items, 0, sizeof(items));
+
+		num_items = info->num_fields;
+
+		Y += 5;
+
+		for (int i = 0 ; i < num_items ; i++)
+		{
+			bool is_change = (y_stricmp(info->fields[i].name, "Change") == 0);
+
+			if (is_change)
+				Y += 10;
+
+			items[i] = new UI_Generalized_Item(X + 100, Y, 120, 22, &info->fields[i]);
+			items[i]->callback(item_callback, this);
+
+			if (is_change && (i+2) < num_items)
+			{
+				change_index  = i;
+				change_widget = items[i];
+			}
+
+			if (change_index >= 0 && i == change_index+1)
+				items[i]->deactivate();
+
+			Y += 30;
+		}
+
+		end();
+	}
+
+	~UI_Generalized_Page()
+	{ }
+
+	void UpdateChange()
+	{
+		if (change_index < 0)
+			return;
+
+		if (items[change_index]->value() == 0)
+		{
+			items[change_index+1]->deactivate();
+			items[change_index+2]->  activate();
+		}
+		else
+		{
+			items[change_index+1]->  activate();
+			items[change_index+2]->deactivate();
+		}
+	}
+
+	int ComputeType() const
+	{
+		int value = 0;
+		
+		for (int i = 0 ; i < num_items ; i++)
+		{
+			if (items[i]->active())
+				value = value | items[i]->Compute();
+		}
+
+		return t_base + value;
+	}
+
+	void DecodeType(int line_type)
+	{
+		line_type -= t_base;
+
+		for (int i = 0 ; i < num_items ; i++)
+		{
+			items[i]->Decode(line_type);
+		}
+
+		UpdateChange();
+	}
+
+	void ResetFields()
+	{
+		for (int i = 0 ; i < num_items ; i++)
+		{
+			items[i]->Reset();
+		}
+	}
+};
+
+
+UI_Generalized_Box::UI_Generalized_Box(int X, int Y, int W, int H, const char *label) :
+    Fl_Group(X, Y, W, H, NULL),
+	num_pages(0),
+	in_update(false)
+{
+	box(FL_FLAT_BOX);
+
+	color(BROWBACK_COL, BROWBACK_COL);
+
+
+	memset(pages, 0, sizeof(pages));
+
+
+	int orig_X = X;
+
+///  X = X + (W - MIN_BROWSER_W);
+
+
+	Y += 10;
+
+	Fl_Box *title = new Fl_Box(X + 30, Y, W - 114, 22+KF*4, label);
+	title->labelsize(20+KF*4);
+
+
+	Fl_Button *hide_button = new Fl_Button(X + 8, Y+2, 22, 22, "X");
+	hide_button->callback(hide_callback, this);
+	hide_button->labelsize(14);
+
+	Y += title->h() + 6;
+
+
+	no_boom = new Fl_Box(FL_NO_BOX, X + 2, Y + 40, W - 60, 60,
+						 "This requires BOOM\n(or a compatible port)");
+	no_boom->labelsize(18);
+	no_boom->labelcolor(FL_BLUE);
+	no_boom->align(FL_ALIGN_INSIDE);
+
+
+	Y += 10;
+
+	category = new Fl_Choice(X + 40, Y, 170, 30);
+	category->callback(cat_callback, this);
+	category->textsize(16);
+
+
+	end();
+
+
+	// resize box
+
+	Fl_Box * rs_box = new Fl_Box(FL_NO_BOX, orig_X + W - 10, Y + H - 10, 8, 8, NULL);
+
+	resizable(rs_box);
+}
+
+
+UI_Generalized_Box::~UI_Generalized_Box()
+{
+	// nothing needed
+}
+
+
+void UI_Generalized_Box::Populate()
+{
+	if (! game_info.gen_types)
+	{
+		no_boom->show();
+
+		category->hide();
+
+		for (int i = 0 ; i < num_pages ; i++)
+			pages[i]->hide();
+	}
+	else
+	{
+		// we only create the pages once
+		// [ not strictly correct, but the generalized types never change ]
+
+		if (! pages[0])
+			CreatePages();
+
+		no_boom->hide();
+
+		category->show();
+
+		for (int i = 0 ; i < num_pages ; i++)
+		{
+			if (i == category->value() - 1)
+				pages[i]->show();
+			else
+				pages[i]->hide();
+		}
+	}
+
+	redraw();
+}
+
+
+void UI_Generalized_Box::CreatePages()
+{
+	memset(pages, 0, sizeof(pages));
+
+	num_pages = 0;
+
+	category->clear();
+
+	category->add("NONE");
+
+	int X = x();  /// + (w() - MIN_BROWSER_W);
+
+	for (int i = 0 ; i < num_gen_linetypes ; i++)
+	{
+		const generalized_linetype_t *info = &gen_linetypes[i];
+
+		category->add(info->name);
+
+		pages[i] = new UI_Generalized_Page(X + 10, y() + 100, 230, 300, info);
+		pages[i]->callback(edit_callback, this);
+
+		add(pages[i]);
+
+		num_pages += 1;
+	}
+
+	category->value(0);
+}
+
+
+int UI_Generalized_Box::ComputeType() const
+{
+	int cur_page = category->value();
+
+	if (cur_page == 0)
+		return 0;
+
+	return pages[cur_page - 1]->ComputeType();
+}
+
+
+void UI_Generalized_Box::UpdateGenType(int line_type)
+{
+	if (in_update)
+		return;
+
+	if (no_boom->visible() || num_pages == 0)
+		return;
+
+	int new_page = -1;
+
+	for (int i = 0 ; i < num_pages ; i++)
+	{
+		if (pages[i]->t_base <= line_type && line_type < pages[i]->t_base + pages[i]->t_length)
+		{
+			new_page = i;
+			break;
+		}
+	}
+
+	if (new_page < 0)
+	{
+		for (int k = 0 ; k < num_pages ; k++)
+			pages[k]->ResetFields();
+
+		if (category->value() != 0)
+		{
+			category->value(0);
+			Populate();
+		}
+
+		return;
+	}
+
+	if (category->value() != new_page + 1)
+	{
+		category->value(new_page + 1);
+		Populate();
+	}
+
+	pages[new_page]->DecodeType(line_type);
+}
+
+
+void UI_Generalized_Box::hide_callback(Fl_Widget *w, void *data)
+{
+	main_win->ShowBrowser(0);
+}
+
+void UI_Generalized_Box::cat_callback(Fl_Widget *w, void *data)
+{
+	UI_Generalized_Box *box = (UI_Generalized_Box *)data;
+
+	int new_page = box->category->value() - 1;
+
+	for (int i = 0 ; i < box->num_pages ; i++)
+	{
+		if (i == new_page)
+			box->pages[i]->show();
+		else
+			box->pages[i]->hide();
+	}
+
+	edit_callback(w, (void *)box);
+
+	box->redraw();
+}
+
+void UI_Generalized_Box::edit_callback(Fl_Widget *w, void *data)
+{
+	UI_Generalized_Box *box = (UI_Generalized_Box *)data;
+
+	if (box->no_boom->visible() || box->num_pages == 0)
+		return;
+
+	box->in_update = true;  // prevent some useless work
+	{
+		int line_type = box->ComputeType();
+
+		main_win->BrowsedItem('L', line_type, "", 0);
+	}
+	box->in_update = false;
 }
 
 
@@ -921,9 +1354,6 @@ UI_Browser::UI_Browser(int X, int Y, int W, int H, const char *label) :
     Fl_Group(X, Y, W, H, label),
 	active(2)
 {
-	end(); // cancel begin() in Fl_Group constructor
-
-
 	// create each browser box
 
 	const char *mode_letters = "TFOLS";
@@ -941,16 +1371,21 @@ UI_Browser::UI_Browser(int X, int Y, int W, int H, const char *label) :
 	{
 		browsers[i] = new UI_Browser_Box(X, Y, W, H, mode_titles[i], mode_letters[i]);
 
-		add(browsers[i]);
-
 		if (i != active)
 			browsers[i]->hide();
 	}
+
+	gen_box = new UI_Generalized_Box(X, Y, W, H, "Generalized");
+	gen_box->hide();
+
+
+	end();
 }
 
 
 UI_Browser::~UI_Browser()
 {
+	// nothing needed
 }
 
 
@@ -961,21 +1396,22 @@ void UI_Browser::Populate()
 		browsers[i]->Populate();
 	}
 
+	gen_box->Populate();
+
 	// setup the categories
 
 	char letters[64];
 
-	const char *tex_cats = M_TextureCategoryString(letters);
-
+	const char *tex_cats = M_TextureCategoryString(letters, false);
 	browsers[0]->SetCategories(tex_cats, letters);
-	browsers[1]->SetCategories(tex_cats, letters);
+
+	const char *flat_cats = M_TextureCategoryString(letters, true);
+	browsers[1]->SetCategories(flat_cats, letters);
 
 	const char *thing_cats = M_ThingCategoryString(letters);
-
 	browsers[2]->SetCategories(thing_cats, letters);
 
 	const char *line_cats = M_LineCategoryString(letters);
-
 	browsers[3]->SetCategories(line_cats, letters);
 
 	// TODO: sector_cats
@@ -989,11 +1425,25 @@ void UI_Browser::SetActive(int new_active)
 	if (new_active == active)
 		return;
 
-	browsers[active]->hide();
+	if (active < ACTIVE_GENERALIZED)
+		browsers[active]->hide();
+	else
+		gen_box->hide();
 
 	active = new_active;
 
-	browsers[active]->show();
+	if (active < ACTIVE_GENERALIZED)
+	{
+		browsers[active]->show();
+		browsers[active]->RecentUpdate();
+	}
+	else
+	{
+		gen_box->show();
+	}
+
+	if (new_active == ACTIVE_GENERALIZED)
+		main_win->tile->MinimiseRight();
 }
 
 
@@ -1007,31 +1457,32 @@ void UI_Browser::ChangeMode(char new_mode)
 		case 'L': SetActive(3); break;  // LINE TYPES
 		case 'S': SetActive(4); break;  // SECTOR TYPES
 
-		default:
-			return;
+		case 'G': SetActive(ACTIVE_GENERALIZED); break;
+
+		default: break;
 	}
 }
 
 
-void UI_Browser::NewEditMode(char edit_mode)
+void UI_Browser::NewEditMode(obj_type_e edit_mode)
 {
 	switch (edit_mode)
 	{
-		case 'l':
+		case OBJ_LINEDEFS:
 			// if on LINE TYPES, stay there
 			// otherwise go to TEXTURES
-			if (active != 3)
+			if (! (active == 3 || active == ACTIVE_GENERALIZED))
 				SetActive(0);
 			break;
 
-		case 's':
+		case OBJ_SECTORS:
 			// if on SECTOR TYPES, stay there
 			// otherwise go to FLATS
 			if (active != 4)
 				SetActive(1);
 			break;
 
-		case 't':
+		case OBJ_THINGS:
 			SetActive(2);
 			break;
 
@@ -1044,40 +1495,46 @@ void UI_Browser::NewEditMode(char edit_mode)
 
 void UI_Browser::CycleCategory(int dir)
 {
-	browsers[active]->CycleCategory(dir);
+	if (active < ACTIVE_GENERALIZED)
+	{
+		browsers[active]->CycleCategory(dir);
+	}
 }
 
 
 void UI_Browser::ClearSearchBox()
 {
-	browsers[active]->ClearSearchBox();
+	if (active < ACTIVE_GENERALIZED)
+	{
+		browsers[active]->ClearSearchBox();
+	}
+
+	// idea : reset generalized info
 }
 
 
 void UI_Browser::Scroll(int delta)
 {
-	browsers[active]->Scroll(delta);
+	if (active < ACTIVE_GENERALIZED)
+	{
+		browsers[active]->Scroll(delta);
+	}
 }
 
 
 void UI_Browser::RecentUpdate()
 {
-	if (! visible())
-		return;
-	
-	UI_Browser_Box *box = browsers[active];
+	if (active < ACTIVE_GENERALIZED)
+	{
+		UI_Browser_Box *box = browsers[active];
 
-///	if (box->GetKind() != kind)
-///		return;
-
-	box->RecentUpdate();
+		box->RecentUpdate();
+	}
 }
 
 
-void UI_Browser::ToggleRecent()
+void UI_Browser::ToggleRecent(bool force_recent)
 {
-	bool force_recent = false;
-
 	// show browser if hidden [ and then force the RECENT category ]
 	if (! visible())
 	{
@@ -1086,7 +1543,16 @@ void UI_Browser::ToggleRecent()
 		force_recent = true;
 	}
 
-	browsers[active]->ToggleRecent(force_recent);
+	if (active < ACTIVE_GENERALIZED)
+	{
+		browsers[active]->ToggleRecent(force_recent);
+	}
+}
+
+
+void UI_Browser::UpdateGenType(int line_type)
+{
+	gen_box->UpdateGenType(line_type);
 }
 
 
@@ -1179,15 +1645,17 @@ void UI_Browser::WriteUser(FILE *fp)
 {
 	fprintf(fp, "\n");
 
-	if (visible())
-		fprintf(fp, "open_browser %c\n", browsers[active]->GetKind());
-	else
-		fprintf(fp, "open_browser -\n");
+	fprintf(fp, "open_browser %c\n",
+		(! visible()) ? '-' :
+		(active >= ACTIVE_GENERALIZED) ? 'G' :
+		browsers[active]->GetKind());
 
 	for (int i = 0 ; i < 5 ; i++)
 	{
 		browsers[i]->WriteUser(fp);
 	}
+
+	// generalized box is not saved (not needed)
 }
 
 
