@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2006-2015 Andrew Apted
+//  Copyright (C) 2006-2016 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -28,8 +28,12 @@
 #include "m_game.h"
 #include "r_grid.h"
 #include "im_color.h"
+#include "im_img.h"
 #include "levels.h"
 #include "r_render.h"
+#include "w_flats.h"
+
+#include <algorithm>
 
 
 #define CAMERA_COLOR  fl_rgb_color(255, 192, 255)
@@ -51,6 +55,9 @@ rgb_color_t normal_flat_col  = RGB_MAKE(84, 96, 48);
 rgb_color_t normal_small_col = RGB_MAKE(60, 60, 120);
 
 
+int vertex_radius(double scale);
+
+
 //
 // UI_Canvas Constructor
 //
@@ -207,6 +214,9 @@ void UI_Canvas::DrawEverything()
 
 	if (edit.action == ACT_SELBOX)
 		SelboxDraw();
+
+	if (edit.action == ACT_DRAW_LINE)
+		DrawCurrentLine();
 }
 
 
@@ -218,6 +228,12 @@ void UI_Canvas::DrawMap()
 	fl_color(FL_BLACK);
 	fl_rectf(x(), y(), w(), h());
 
+	if (edit.sector_render_mode && (! grid.shown || grid.mode == 0))
+	{
+		for (int n = 0 ; n < NumSectors ; n++)
+			RenderSector(n);
+	}
+
 	// draw the grid first since it's in the background
 	if (grid.shown)
 	{
@@ -1281,7 +1297,7 @@ void UI_Canvas::DrawSplitLine(int map_x1, int map_y1, int map_x2, int map_y2)
 	fl_line(scr_x1, scr_y1, scr_mx, scr_my);
 	fl_line(scr_x2, scr_y2, scr_mx, scr_my);
 
-	int size = 7;
+	int size = (grid.Scale >= 5.0) ? 11 : (grid.Scale >= 1.0) ? 9 : 7;
 
 	fl_pie(scr_mx - size/2, scr_my - size/2, size, size, 0, 360);
 }
@@ -1393,33 +1409,34 @@ void UI_Canvas::DrawCamera()
 }
 
 
-void UI_Canvas::DrawSnapMarker()
+void UI_Canvas::DrawCurrentLine()
 {
-	// show where the closest snap position is
-	// [ Not currently used ]
-
-	if (! grid.snap)
+	if (edit.drawing_from < 0)
 		return;
 
-	int map_x = grid.SnapX(edit.map_x);
-	int map_y = grid.SnapY(edit.map_y);
+	int new_x = grid.SnapX(edit.map_x);
+	int new_y = grid.SnapY(edit.map_y);
 
-	if (! Vis(map_x, map_y, 20))
-		return;
+	// should draw a vertex?
+	if (highlight.valid())
+	{
+		SYS_ASSERT(highlight.type == OBJ_VERTICES);
 
-	int size = 8;
+		new_x = Vertices[highlight.num]->x;
+		new_y = Vertices[highlight.num]->y;
+	}
+	else if (split_ld < 0)
+	{
+		fl_color(FL_GREEN);
 
-	int scr_x = SCREENX(map_x);
-	int scr_y = SCREENY(map_y);
+		DrawVertex(new_x, new_y, vertex_radius(grid.Scale));
+	}
 
-	fl_color(fl_rgb_color(160,80,160));
+	fl_color(RED);
 
-	fl_pie(scr_x - size/2, scr_y - size/2, size, size, 0, 360);
+	const Vertex * v = Vertices[edit.drawing_from];
 
-	int scr_x2 = SCREENX(edit.map_x);
-	int scr_y2 = SCREENY(edit.map_y);
-
-	fl_line(scr_x, scr_y, scr_x2, scr_y2);
+	DrawKnobbyLine(v->x, v->y, new_x, new_y);
 }
 
 
@@ -1512,16 +1529,6 @@ void UI_Canvas::DragDelta(int *dx, int *dy)
 	*dx = drag_cur_x - drag_start_x;
 	*dy = drag_cur_y - drag_start_y;
 
-	// no real dragging until mouse distance has moved
-	int scr_dx = SCREENX(drag_cur_x) - SCREENX(drag_start_x);
-	int scr_dy = SCREENY(drag_cur_y) - SCREENY(drag_start_y);
-
-	if (abs(scr_dx) < 6 && abs(scr_dy) < 6)  // TODO: CONFIG ITEM
-	{
-		*dx = *dy = 0;
-		return;
-	}
-
 	if (grid.snap)
 	{
 		int focus_x = drag_focus_x + *dx;
@@ -1566,7 +1573,7 @@ void UI_Canvas::ScaleUpdate(int map_x, int map_y, keycode_t mod)
 	int dx2 = scale_start_x - scale_param.mid_x;
 	int dy2 = scale_start_y - scale_param.mid_y;
 
-	bool any_aspect = (mod & MOD_ALT)     ? true : false;
+	bool any_aspect = (mod & MOD_SHIFT)   ? true : false;
 	bool rotate     = (mod & MOD_COMMAND) ? true : false;
 
 	scale_param.rotate = 0;
@@ -1603,5 +1610,313 @@ void UI_Canvas::ScaleUpdate(int map_x, int map_y, keycode_t mod)
 }
 
 
+//------------------------------------------------------------------------
+
+
+// this represents a segment of a linedef bounding a sector.
+struct sector_edge_t
+{
+	const LineDef * line;
+
+	// coordinates mapped to screen space, not clipped
+	int scr_x1, scr_y1;
+	int scr_x2, scr_y2;
+
+	// has the line been flipped (coordinates were swapped) ?
+	short flipped;
+
+	// what side this edge faces (SIDE_LEFT or SIDE_RIGHT)
+	short side;
+
+	// clipped vertical range, inclusive
+	short y1, y2;
+
+	// computed X value
+	float x;
+
+	void CalcX(short y)
+	{
+		x = scr_x1 + (scr_x2 - scr_x1) * (float)(y - scr_y1) / (float)(scr_y2 - scr_y1);
+	}
+
+	struct CMP_Y
+	{
+		inline bool operator() (const sector_edge_t &A, const sector_edge_t& B) const
+		{
+			return A.scr_y1 < B.scr_y1;
+		}
+	};
+
+	struct CMP_X
+	{
+		inline bool operator() (const sector_edge_t *A, const sector_edge_t *B) const
+		{
+			// NULL is always > than a valid pointer
+
+			if (A == NULL)
+				return false;
+
+			if (B == NULL)
+				return true;
+
+			return A->x < B->x;
+		}
+	};
+};
+
+
+void UI_Canvas::RenderSector(int num)
+{
+///  fprintf(stderr, "RenderSector %d\n", num);
+
+	rgb_color_t light_col = SectorLightColor(Sectors[num]->light);
+
+	const char * tex_name = NULL;
+
+	Img_c * img = NULL;
+
+	if (edit.sector_render_mode == SREND_Lighting)
+	{
+		fl_color(light_col); 
+	}
+	else
+	{
+		if (edit.sector_render_mode == SREND_Ceiling)
+			tex_name = Sectors[num]->CeilTex();
+		else
+			tex_name = Sectors[num]->FloorTex();
+
+		if (is_sky(tex_name))
+		{
+			fl_color(palette[game_info.sky_color]);
+		}
+		else
+		{
+			img = W_GetFlat(tex_name);
+
+			if (! img)
+			{
+				img = IM_UnknownTex();
+			}
+		}
+	}
+
+	img_pixel_t *wbuf = img ? img->wbuf() : NULL;
+
+	int tw = img ? img->width()  : 1;
+	int th = img ? img->height() : 1;
+
+	// verify size is at least 64x64
+	if (img && (tw < 64 || th < 64))
+	{
+		fl_color(palette[game_info.missing_color]);
+
+		img = NULL;
+	}
+
+
+	/*** Part 1 : visit linedefs and create edges ***/
+
+
+	std::vector<sector_edge_t> edgelist;
+
+	short min_y = 32767;
+	short max_y = 0;
+
+
+	for (int n = 0 ; n < NumLineDefs ; n++)
+	{
+		const LineDef *L = LineDefs[n];
+
+		if (! L->TouchesSector(num))
+			continue;
+
+		// ignore 2S lines with same sector on both sides
+		if (L->WhatSector(SIDE_LEFT) == L->WhatSector(SIDE_RIGHT))
+			continue;
+
+		sector_edge_t edge;
+
+		edge.scr_x1 = SCREENX(L->Start()->x);
+		edge.scr_y1 = SCREENY(L->Start()->y);
+		edge.scr_x2 = SCREENX(L->End()->x);
+		edge.scr_y2 = SCREENY(L->End()->y);
+
+		// completely above or below the screen?
+		if (MAX(edge.scr_y1, edge.scr_y2) < y())
+			continue;
+
+		if (MIN(edge.scr_y1, edge.scr_y2) >= y() + h())
+			continue;
+				
+		// skip horizontal lines
+		if (edge.scr_y1 == edge.scr_y2)
+			continue;
+
+		edge.flipped = 0;
+
+		if (edge.scr_y1 > edge.scr_y2)
+		{
+			std::swap(edge.scr_x1, edge.scr_x2);
+			std::swap(edge.scr_y1, edge.scr_y2);
+
+			edge.flipped = 1;
+		}
+
+		// compute usable range, clipping to screen
+		edge.y1 = MAX(edge.scr_y1, y());
+		edge.y2 = MIN(edge.scr_y2, y() + h() - 1);
+
+		// this probably cannot happen....
+		if (edge.y2 < edge.y1)
+			continue;
+
+		min_y = MIN(min_y, edge.y1);
+		max_y = MAX(max_y, edge.y2);
+
+		// compute side
+		bool is_right = (L->WhatSector(SIDE_LEFT) == num);
+
+		if (edge.flipped) is_right = !is_right;
+
+		edge.side = is_right ? SIDE_RIGHT : SIDE_LEFT;
+
+/*
+fprintf(stderr, "Line %d  mapped coords (%d %d) .. (%d %d)  flipped:%d  sec:%d/%d\n",
+n, edge.scr_x1, edge.scr_y1, edge.scr_x2, edge.scr_y2, edge.flipped,
+L->WhatSector(SIDE_RIGHT), L->WhatSector(SIDE_LEFT));
+*/
+
+		// add the edge
+		edge.line = L;
+
+		edgelist.push_back(edge);
+	}
+
+	if (edgelist.empty())
+		return;
+
+	// sort edges into vertical order (i.e. by scr_y1)
+
+	std::sort(edgelist.begin(), edgelist.end(), sector_edge_t::CMP_Y());
+
+
+	/*** Part 2 : traverse edge list and render spans ***/
+
+
+	unsigned int next_edge = 0;
+
+	u8_t * line_rgb = new u8_t[3 * (w() + 4)];
+
+	std::vector<sector_edge_t *> active_edges;
+
+	unsigned int i;
+
+	// visit each screen row
+	for (short y = min_y ; y <= max_y ; y++)
+	{
+		// remove old edges from active list
+		for (i = 0 ; i < active_edges.size() ; i++)
+		{
+			if (y > active_edges[i]->y2)
+				active_edges[i] = NULL;
+		}
+
+		// add new edges from active list
+		for ( ; next_edge < edgelist.size() && y == edgelist[next_edge].y1 ; next_edge++)
+		{
+			active_edges.push_back(&edgelist[next_edge]);
+		}
+
+///  fprintf(stderr, "  active @ y=%d --> %d\n", y, (int)active_edges.size());
+
+		if (active_edges.empty())
+			continue;
+
+		// sort active edges by X value
+		// [ also puts NULL entries at end, making easy to remove them ]
+
+		for (i = 0 ; i < active_edges.size() ; i++)
+		{
+			if (active_edges[i])
+				active_edges[i]->CalcX(y);
+		}
+
+		std::sort(active_edges.begin(), active_edges.end(), sector_edge_t::CMP_X());
+
+		while (active_edges.size() > 0 && active_edges.back() == NULL)
+			active_edges.pop_back();
+
+		// compute spans
+		
+		for (unsigned int i = 1 ; i < active_edges.size() ; i++)
+		{
+			const sector_edge_t * E1 = active_edges[i - 1];
+			const sector_edge_t * E2 = active_edges[i];
+#if 1
+			if (E1 == NULL || E2 == NULL)
+				BugError("RenderSector: did not delete NULLs properly!");
+#endif
+
+///  fprintf(stderr, "E1 @ x=%1.2f side=%d  |  E2 @ x=%1.2f side=%d\n",
+///  E1->x, E1->side, E2->x, E2->side);
+
+			if (! (E1->side == SIDE_RIGHT && E2->side == SIDE_LEFT))
+				continue;
+
+			// treat lines without a right side as dead
+			if (E1->line->right < 0) continue;
+			if (E2->line->right < 0) continue;
+
+			int x1 = floor(E1->x);
+			int x2 = floor(E2->x);
+
+			// completely off the screen?
+			if (x2 < x() || x1 >= x() + w())
+				continue;
+
+			// clip span to screen
+			x1 = MAX(x1, x());
+			x2 = MIN(x2, x() + w() - 1);
+
+			// this probably cannot happen....
+			if (x2 < x1) continue;
+
+///  fprintf(stderr, "  span : y=%d  x=%d..%d\n", y, x1, x2);
+
+			// solid color?
+			if (! img)
+			{
+				fl_rectf(x1, y, x2 - x1 + 1, 1);
+				continue;
+			}
+
+			int x = x1;
+			int span_w = x2 - x1 + 1;
+
+			u8_t *dest = line_rgb;
+			u8_t *dest_end = line_rgb + span_w * 3;
+
+			int ty = (0 - MAPY(y)) & 63;
+
+			for (; dest < dest_end ; dest += 3, x++)
+			{
+				// TODO : be nice to optimize the next line
+				int tx = MAPX(x) & 63;
+
+				rgb_color_t col = palette[wbuf[ty * tw + tx]];
+
+				dest[0] = RGB_RED(col);
+				dest[1] = RGB_GREEN(col);
+				dest[2] = RGB_BLUE(col);
+			}
+
+			fl_draw_image(line_rgb, x1, y, span_w, 1);
+		}
+	}
+
+	delete[] line_rgb;
+}
+
 //--- editor settings ---
 // vi:ts=4:sw=4:noexpandtab
