@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2015 Andrew Apted
+//  Copyright (C) 2015-2016 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -22,6 +22,8 @@
 #include "ui_window.h"
 
 #include "e_path.h"  // GoToObject
+#include "levels.h"  // Selection_Clear
+#include "m_config.h"	// gui_scheme
 #include "m_game.h"
 #include "w_rawdef.h"
 
@@ -262,6 +264,10 @@ public:
 
 //------------------------------------------------------------------------
 
+
+#define HIDE_BG  (gui_scheme == 2 ? FL_DARK3 : FL_DARK1)
+
+
 UI_FindAndReplace::UI_FindAndReplace(int X, int Y, int W, int H) :
 	Fl_Group(X, Y, W, H, NULL),
 	find_numbers(new number_group_c),
@@ -278,7 +284,13 @@ UI_FindAndReplace::UI_FindAndReplace(int X, int Y, int W, int H) :
 	Fl_Group *grp1 = new Fl_Group(X, Y, W, 210);
 	grp1->box(FL_UP_BOX);
 	{
-		Fl_Box *title = new Fl_Box(X + 50, Y + 10, W - 60, 30, "Find and Replace");
+		Fl_Button *hide_button = new Fl_Button(X + 8, Y + 12, 22, 22, "X");
+		hide_button->color(HIDE_BG, HIDE_BG);
+		hide_button->labelsize(14);
+		hide_button->callback(hide_callback, this);
+
+
+		Fl_Box *title = new Fl_Box(X + 60, Y + 10, W - 70, 30, "Find and Replace");
 		title->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
 		title->labelsize(18+KF*4);
 
@@ -401,6 +413,12 @@ UI_FindAndReplace::~UI_FindAndReplace()
 { }
 
 
+void UI_FindAndReplace::hide_callback(Fl_Widget *w, void *data)
+{
+	main_win->HideSpecialPanel();
+}
+
+
 void UI_FindAndReplace::UpdateWhatColor()
 {
 	switch (what->value())
@@ -423,13 +441,15 @@ void UI_FindAndReplace::UpdateWhatFilters()
 	if (test) (w)->show(); else (w)->hide();
 
 	// common stuff
-	if (x != 0)
-		tag_input->activate();
-	else
+	if (x == 0 && Level_format != MAPF_Hexen)
 	{
 		tag_input->deactivate();
 		tag_input->value("");
 	}
+	else
+	{
+		tag_input->activate();
+	}
 
 	// thing stuff
 	SHOW_WIDGET_IF(o_easy,   x == 0);
@@ -456,7 +476,7 @@ void UI_FindAndReplace::UpdateWhatFilters()
 #undef SHOW_WIDGET_IF
 
 	// vanilla DOOM : always hide SP and COOP flags
-	if (x == 0 && ! game_info.coop_dm_flags)
+	if (x == 0 && ! game_info.coop_dm_flags && Level_format != MAPF_Hexen)
 	{
 		  o_sp->hide();
 		o_coop->hide();
@@ -994,16 +1014,9 @@ bool UI_FindAndReplace::FindNext()
 	if (cur_obj.type != edit.mode)
 	{
 		Editor_ChangeMode_Raw(cur_obj.type);
-
-		// this clears the selection
-		edit.Selected->change_type(edit.mode);
-	}
-	else
-	{
-		edit.Selected->clear_all();
 	}
 
-	edit.RedrawMap = 1;
+	Selection_Clear();
 
 
 	bool is_first = cur_obj.is_nil();
@@ -1197,7 +1210,7 @@ void UI_FindAndReplace::DoAll(bool replace)
 		rep_value->do_callback();
 	}
 
-	edit.RedrawMap = 1;
+	RedrawMap();
 }
 
 
@@ -1216,6 +1229,9 @@ bool UI_FindAndReplace::Match_Thing(int idx)
 	if ((T->options & options_mask) != options_value)
 		return false;
 
+	if (Level_format == MAPF_Hexen && ! Filter_Tag(T->tid))
+		return false;
+
 	return true;
 }
 
