@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2015 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -726,7 +726,7 @@ static void get_cur_linedef(Close_obj& closest, int x, int y)
  *  get_split_linedef - determine which linedef would be split if a
  *                      new vertex was added to the given point.
  */
-static void get_split_linedef(Close_obj& closest, int x, int y, int drag_vert)
+static void get_split_linedef(Close_obj& closest, int x, int y, int ignore_vert)
 {
 	// slack in map units
 	int mapslack = 1 + (int)ceil(8.0f / grid.Scale);
@@ -740,8 +740,7 @@ static void get_split_linedef(Close_obj& closest, int x, int y, int drag_vert)
 	{
 		LineDef *L = LineDefs[n];
 
-		// ignore the dragging vertex (drag_vert can be -1 for none)
-		if (L->start == drag_vert || L->end == drag_vert)
+		if (L->start == ignore_vert || L->end == ignore_vert)
 			continue;
 
 		int x1 = L->Start()->x;
@@ -876,15 +875,12 @@ static void get_cur_thing(Close_obj& closest, int x, int y)
 /*
  *  get_cur_vertex - determine which vertex is under the pointer
  */
-static void get_cur_vertex(Close_obj& closest, int x, int y, bool snap)
+static void get_cur_vertex(Close_obj& closest, int x, int y)
 {
 	const int screen_pix = vertex_radius(grid.Scale);
 
 	int mapslack = 1 + (int)ceil((4 + screen_pix) / grid.Scale);
 
-	if (snap)
-		mapslack = grid.step / 2 + 1;
-
 	int lx = x - mapslack;
 	int ly = y - mapslack;
 	int hx = x + mapslack;
@@ -914,13 +910,13 @@ static void get_cur_vertex(Close_obj& closest, int x, int y, bool snap)
 
 
 /*
- *  GetCurObject - determine which object is under the pointer
+ *  GetNearObject - determine which object is under the pointer
  * 
  *  Set <o> to point to the object under the pointer (map
  *  coordinates (<x>, <y>). If several objects are close
  *  enough to the pointer, the smallest object is chosen.
  */
-void GetCurObject(Objid& o, obj_type_e objtype, int x, int y, bool snap)
+void GetNearObject(Objid& o, obj_type_e objtype, int x, int y)
 {
 	Close_obj closest;
 
@@ -935,7 +931,7 @@ void GetCurObject(Objid& o, obj_type_e objtype, int x, int y, bool snap)
 
 		case OBJ_VERTICES:
 		{
-			get_cur_vertex(closest, x, y, snap);
+			get_cur_vertex(closest, x, y);
 			o = closest.obj;
 			break;
 		}
@@ -955,7 +951,7 @@ void GetCurObject(Objid& o, obj_type_e objtype, int x, int y, bool snap)
 		}
 
 		default:
-			BugError("GetCurObject: bad objtype %d", (int) objtype);
+			BugError("GetNearObject: bad objtype %d", (int) objtype);
 			break; /* NOT REACHED */
 	}
 }
@@ -971,7 +967,7 @@ void GetSplitLineDef(Objid& o, int x, int y, int drag_vert)
 
 	// don't highlight the line if the new vertex would snap onto
 	// the same coordinate as the start or end of the linedef.
-	// (I tried a bbox test here, but it's bad for axis-aligned lines)
+	// [ I tried a bbox test here, but it's bad for axis-aligned lines ]
 
 	if (o.valid() && grid.snap)
 	{
@@ -985,8 +981,179 @@ void GetSplitLineDef(Objid& o, int x, int y, int drag_vert)
 		{
 			o.clear();
 		}
+
+		// also require snap coordinate be not TOO FAR from the line
+		double len = L->CalcLength();
+
+		double along = AlongDist(snap_x, snap_y, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
+		double  perp =  PerpDist(snap_x, snap_y, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
+
+		if (along < len * 0.1 || along > len * 0.9 || fabs(perp) > len * 0.2)
+		{
+			o.clear();
+		}
 	}
 }
 
+
+void GetSplitLineForDangler(Objid& o, int v_num)
+{
+	Close_obj closest;
+
+	get_split_linedef(closest, Vertices[v_num]->x, Vertices[v_num]->y, v_num);
+
+	o = closest.obj;
+}
+
+
+bool FindClosestCrossPoint(int v1, int v2, cross_state_t *cross)
+{
+	SYS_ASSERT(v1 != v2);
+
+	cross->vert = -1;
+	cross->line = -1;
+
+	const Vertex *VA = Vertices[v1];
+	const Vertex *VB = Vertices[v2];
+
+	int x1 = VA->x;
+	int y1 = VA->y;
+	int x2 = VB->x;
+	int y2 = VB->y;
+
+	int dx = x2 - x1;
+	int dy = y2 - y1;
+
+	// same coords?  (generally should not happen, handle it anyway)
+	if (dx == 0 && dy == 0)
+		return false;
+
+	double length = sqrt(dx*dx + dy*dy);
+
+	double epsilon = 0.4;
+
+	// when zooming out, make it easier to hit a vertex
+	double sk = 1.0 / grid.Scale;
+	double close_dist = 8 * sqrt(sk);
+
+	close_dist = CLAMP(1.2, close_dist, 24.0);
+
+	double best_dist = 9e9;
+
+	/* try all vertices */
+	
+	for (int v = 0 ; v < NumVertices ; v++)
+	{
+		if (v == v1 || v == v2)
+			continue;
+
+		const Vertex * VC = Vertices[v];
+
+		// ignore vertices ar same coordinates as v1 or v2
+		if (VC->x == VA->x && VC->y == VA->y) continue;
+		if (VC->x == VB->x && VC->y == VB->y) continue;
+
+		// is this vertex sitting on the line?
+#if 0
+		if (x1 == x2)
+		{
+			if (VC->x != x1)
+				continue;
+		}
+		else if (y1 == y2)
+		{
+			if (VC->y != y1)
+				continue;
+		}
+		else
+#endif
+		{
+			double perp = PerpDist(VC->x, VC->y, x1,y1, x2,y2);
+
+			if (fabs(perp) > close_dist)
+				continue;
+		}
+
+		double along = AlongDist(VC->x, VC->y, x1,y1, x2,y2);
+
+		if (along < epsilon || along > length - epsilon)
+			continue;
+
+		// yes it is
+
+		if (along < best_dist)
+		{
+			best_dist = along;
+
+			cross->vert = v;
+			cross->line = -1;
+
+			cross->x = VC->x;
+			cross->y = VC->y;
+		}
+	}
+
+
+	/* try all linedefs */
+
+	for (int ld = 0 ; ld < NumLineDefs ; ld++)
+	{
+		const LineDef * L = LineDefs[ld];
+
+		// only need to handle cases where this linedef distinctly crosses
+		// the new line (i.e. start and end are clearly on opposite sides).
+
+		double lx1 = L->Start()->x;
+		double ly1 = L->Start()->y;
+		double lx2 = L->End()->x;
+		double ly2 = L->End()->y;
+
+		double a = PerpDist(lx1,ly1, x1,y1, x2,y2);
+		double b = PerpDist(lx2,ly2, x1,y1, x2,y2);
+
+		if (! ((a < -epsilon && b > epsilon) || (a > epsilon && b < -epsilon)))
+			continue;
+
+		// compute intersection point
+		double l_along = a / (a - b); 
+
+		double ix = lx1 + l_along * (lx2 - lx1);
+		double iy = ly1 + l_along * (ly2 - ly1);
+
+		int new_x = I_ROUND(ix);
+		int new_y = I_ROUND(iy);
+
+		// ensure new vertex does not match the start or end points
+		if (new_x == x1 && new_y == y1) continue;
+		if (new_x == x2 && new_y == y2) continue;
+
+		double along = AlongDist(new_x, new_y,  x1,y1, x2,y2);
+
+		if (along < epsilon || along > length - epsilon)
+			continue;
+
+		// OK, this linedef crosses it
+
+fprintf(stderr, "linedef #%d crosses at (%1.3f %1.3f)  along=%1.3f\n", ld, ix, iy, along);
+
+		// allow vertices to win over a nearby linedef
+		along += close_dist * 2;
+
+		if (along < best_dist)
+		{
+			best_dist = along;
+
+			cross->vert = -1;
+			cross->line = ld;
+
+			cross->x = new_x;
+			cross->y = new_y;
+		}
+	}
+
+
+	return (cross->vert >= 0) || (cross->line >= 0);
+}
+
 //--- editor settings ---
 // vi:ts=4:sw=4:noexpandtab
